@@ -21,7 +21,7 @@ Boston, MA 02111-1307, USA.  */
 
 /* Some output-actions in dsp1600.md need these.  */
 #include "config.h"
-#include <stdio.h>
+#include "system.h"
 #include "rtl.h"
 #include "regs.h"
 #include "hard-reg-set.h"
@@ -33,19 +33,24 @@ Boston, MA 02111-1307, USA.  */
 #include "insn-attr.h"
 #include "tree.h"
 #include "expr.h"
+#include "function.h"
 #include "flags.h"
+#include "ggc.h"
+#include "toplev.h"
+#include "recog.h"
+#include "tm_p.h"
 
-char *text_seg_name;
-char *rsect_text;
-char *data_seg_name;
-char *rsect_data;
-char *bss_seg_name;
-char *rsect_bss;
-char *const_seg_name;
-char *rsect_const;
+const char *text_seg_name;
+const char *rsect_text;
+const char *data_seg_name;
+const char *rsect_data;
+const char *bss_seg_name;
+const char *rsect_bss;
+const char *const_seg_name;
+const char *rsect_const;
 
-char *chip_name;
-char *save_chip_name;
+const char *chip_name;
+const char *save_chip_name;
 
 /* Save the operands of a compare. The 16xx has not lt or gt, so
    in these cases we swap the operands and reverse the condition */
@@ -54,10 +59,10 @@ rtx dsp16xx_compare_op0;
 rtx dsp16xx_compare_op1;
 struct rtx_def *(*dsp16xx_compare_gen)();
 
-static char *fp;
-static char *sp;
-static char *rr;
-static char *a1h;
+static const char *fp;
+static const char *sp;
+static const char *rr;
+static const char *a1h;
 
 struct dsp16xx_frame_info current_frame_info;
 struct dsp16xx_frame_info zero_frame_info;
@@ -85,14 +90,14 @@ rtx dsp16xx_ashlhi3_libcall = (rtx) 0;
 rtx dsp16xx_ucmphi2_libcall = (rtx) 0;
 rtx dsp16xx_lshrhi3_libcall = (rtx) 0;
 
-char *himode_reg_name[] = HIMODE_REGISTER_NAMES;
+static const char *const himode_reg_name[] = HIMODE_REGISTER_NAMES;
 
 #define SHIFT_INDEX_1   0
 #define SHIFT_INDEX_4   1
 #define SHIFT_INDEX_8   2
 #define SHIFT_INDEX_16  3
 
-static char *ashift_right_asm[] = 
+static const char *const ashift_right_asm[] = 
 {
   "%0=%0>>1",
   "%0=%0>>4",
@@ -100,7 +105,7 @@ static char *ashift_right_asm[] =
   "%0=%0>>16"
 };
 
-static char *ashift_right_asm_first[] = 
+static const char *const ashift_right_asm_first[] = 
 {
   "%0=%1>>1",
   "%0=%1>>4",
@@ -108,7 +113,7 @@ static char *ashift_right_asm_first[] =
   "%0=%1>>16"
 };
 
-static char *ashift_left_asm[] = 
+static const char *const ashift_left_asm[] = 
 {
   "%0=%0<<1",
   "%0=%0<<4",
@@ -116,7 +121,7 @@ static char *ashift_left_asm[] =
   "%0=%0<<16"
 };
 
-static char *ashift_left_asm_first[] = 
+static const char *const ashift_left_asm_first[] = 
 {
   "%0=%1<<1",
   "%0=%1<<4",
@@ -124,7 +129,7 @@ static char *ashift_left_asm_first[] =
   "%0=%1<<16"
 };
 
-static char *lshift_right_asm[] = 
+static const char *const lshift_right_asm[] = 
 {
   "%0=%0>>1\n\t%0=%b0&0x7fff",
   "%0=%0>>4\n\t%0=%b0&0x0fff",
@@ -132,7 +137,7 @@ static char *lshift_right_asm[] =
   "%0=%0>>16\n\t%0=%b0&0x0000"
 };
 
-static char *lshift_right_asm_first[] = 
+static const char *const lshift_right_asm_first[] = 
 {
   "%0=%1>>1\n\t%0=%b0&0x7fff",
   "%0=%1>>4\n\t%0=%b0&0x0fff",
@@ -140,10 +145,12 @@ static char *lshift_right_asm_first[] =
   "%0=%1>>16\n\t%0=%b0&0x0000"
 };
 
+static int reg_save_size PARAMS ((void));
+
 int 
 hard_regno_mode_ok (regno, mode)
-int regno;
-enum machine_mode mode;
+     int regno;
+     enum machine_mode mode;
 {
   switch ((int) mode)
     {
@@ -184,7 +191,7 @@ enum machine_mode mode;
 
 enum reg_class
 dsp16xx_reg_class_from_letter (c)
-int c;
+     int c;
 {
   switch (c)
     {
@@ -270,7 +277,7 @@ int c;
 
 int 
 regno_reg_class(regno)
-int regno;
+     int regno;
 {
   switch (regno)
     {
@@ -343,16 +350,16 @@ int regno;
 
 int
 class_max_nregs(class, mode)
-enum reg_class class;
-enum machine_mode mode;
+     enum reg_class class ATTRIBUTE_UNUSED;
+     enum machine_mode mode;
 {
     return (GET_MODE_SIZE(mode));
 }
 
 enum reg_class
 limit_reload_class (mode, class)
-enum machine_mode mode;
-enum reg_class class;
+     enum machine_mode mode;
+     enum reg_class class;
 {
   switch ((int) class)
     {
@@ -549,7 +556,7 @@ enum reg_class class;
 
 int
 dsp16xx_register_move_cost (from, to)
-enum reg_class from, to;
+     enum reg_class from, to;
 {
 #if 0
   if (from == NO_REGS || to == NO_REGS || (from == to))
@@ -868,15 +875,15 @@ secondary_reload_class (class, mode, in)
 
 int
 symbolic_address_operand (op, mode)
-rtx op;
-enum machine_mode mode;
+     rtx op;
+     enum machine_mode mode ATTRIBUTE_UNUSED;
 {
-    return (symbolic_address_p (op));
-
+  return (symbolic_address_p (op));
 }
 
-int symbolic_address_p (op)
-rtx op;
+int
+symbolic_address_p (op)
+     rtx op;
 {
   switch (GET_CODE (op))
     {
@@ -902,16 +909,16 @@ rtx op;
 
 int
 Y_address_operand (op, mode)
-rtx op;
-enum machine_mode mode;
+     rtx op;
+     enum machine_mode mode;
 {
-   return (memory_address_p (mode, op) && !symbolic_address_p (op));
+  return (memory_address_p (mode, op) && !symbolic_address_p (op));
 }	     
 
 int
 sp_operand (op, mode)
-rtx op;
-enum machine_mode mode;
+     rtx op;
+     enum machine_mode mode ATTRIBUTE_UNUSED;
 {
     return (GET_CODE (op) == PLUS
 	    && (XEXP (op, 0) == stack_pointer_rtx
@@ -921,8 +928,8 @@ enum machine_mode mode;
 
 int
 sp_operand2 (op, mode)
-rtx op;
-enum machine_mode mode;
+     rtx op;
+     enum machine_mode mode ATTRIBUTE_UNUSED;
 {
   if ((GET_CODE (op) == PLUS 
        && (XEXP (op, 0) == stack_pointer_rtx
@@ -942,16 +949,16 @@ enum machine_mode mode;
 
 int
 nonmemory_arith_operand (op, mode)
-rtx op;
-enum machine_mode mode;
+     rtx op;
+     enum machine_mode mode;
 {
   return (immediate_operand (op, mode) || arith_reg_operand (op, mode));
 }
 
 int
 arith_reg_operand (op, mode)
-rtx op;
-enum machine_mode mode;
+     rtx op;
+     enum machine_mode mode;
 {
   return (register_operand (op, mode)
 	  && (GET_CODE (op) != REG
@@ -962,8 +969,8 @@ enum machine_mode mode;
 
 int
 call_address_operand (op, mode)
-rtx op;
-enum machine_mode mode;
+     rtx op;
+     enum machine_mode mode ATTRIBUTE_UNUSED;
 {
     if (symbolic_address_p (op) || REG_P(op))
     {
@@ -986,7 +993,7 @@ dsp16xx_comparison_operator (op, mode)
 
 void
 notice_update_cc(exp)
-rtx exp;
+     rtx exp;
 {
     if (GET_CODE (exp) == SET)
     {
@@ -1102,8 +1109,9 @@ dsp16xx_makes_calls ()
   return 0;
 }
 
-long compute_frame_size (size)
-int size;
+long
+compute_frame_size (size)
+     int size;
 {
   long total_size;
   long var_size;
@@ -1142,7 +1150,7 @@ int size;
 
 int
 dsp16xx_call_saved_register (regno)
-int regno;
+     int regno;
 {
   return (regs_ever_live[regno] && !call_used_regs[regno] &&
 	  !IS_YBASE_REGISTER_WINDOW(regno));
@@ -1167,8 +1175,8 @@ ybase_regs_ever_used ()
 
 void 
 function_prologue (file, size)
-FILE *file;
-int  size;
+     FILE *file;
+     int  size;
 {
   int regno;
   long total_size;
@@ -1180,14 +1188,14 @@ int  size;
   total_size = compute_frame_size (size);
   
   fprintf( file, "\t/* FUNCTION PROLOGUE: */\n" );
-  fprintf (file, "\t/* total=%d, vars= %d, regs= %d, args=%d, extra= %d */\n",
+  fprintf (file, "\t/* total=%ld, vars= %ld, regs= %d, args=%d, extra= %ld */\n",
 	   current_frame_info.total_size,
 	   current_frame_info.var_size,
 	   current_frame_info.reg_size,
 	   current_function_outgoing_args_size,
 	   current_frame_info.extra_size);
   
-  fprintf (file, "\t/* fp save offset= %d, sp save_offset= %d */\n\n",
+  fprintf (file, "\t/* fp save offset= %ld, sp save_offset= %ld */\n\n",
 	   current_frame_info.fp_save_offset,
 	   current_frame_info.sp_save_offset);
   /* Set up the 'ybase' register window. */
@@ -1215,7 +1223,7 @@ int  size;
       else
         {
 	  if(SMALL_INTVAL(current_frame_info.var_size) && ((current_frame_info.var_size & 0x8000) == 0))
-	    fprintf (file, "\t%s=%d\n\t*%s++%s\n", reg_names[REG_J], current_frame_info.var_size, sp, reg_names[REG_J]);
+	    fprintf (file, "\t%s=%ld\n\t*%s++%s\n", reg_names[REG_J], current_frame_info.var_size, sp, reg_names[REG_J]);
 	  else
 	    fatal ("Stack size > 32k");
 	}
@@ -1242,7 +1250,7 @@ int  size;
       else
         {
 	  if(SMALL_INTVAL(current_frame_info.args_size) && ((current_frame_info.args_size & 0x8000) == 0))
-	    fprintf (file, "\t%s=%d\n\t*%s++%s\n", reg_names[REG_J], current_frame_info.args_size, sp, reg_names[REG_J]);
+	    fprintf (file, "\t%s=%ld\n\t*%s++%s\n", reg_names[REG_J], current_frame_info.args_size, sp, reg_names[REG_J]);
 	  else
 	    fatal ("Stack size > 32k");
 	}
@@ -1252,7 +1260,7 @@ int  size;
     {
       fprintf( file, "\t%s=%s\n", a1h, sp );
       fprintf( file, "\t%s=%s\n", fp, a1h );  /* Establish new base frame */
-      fprintf( file, "\t%s=%d\n", reg_names[REG_J], -total_size);
+      fprintf( file, "\t%s=%ld\n", reg_names[REG_J], -total_size);
       fprintf( file, "\t*%s++%s\n", fp, reg_names[REG_J]);
     }
   
@@ -1288,11 +1296,13 @@ init_emulation_routines ()
 }
 void
 function_epilogue (file, size)
-FILE *file;
-int   size;
+     FILE *file;
+     int size ATTRIBUTE_UNUSED;
 {
   int regno;
+#if OLD_REGISTER_SAVE  
   int initial_stack_dec = 0;
+#endif
   
   fp = reg_names[FRAME_POINTER_REGNUM];
   sp = reg_names[STACK_POINTER_REGNUM];
@@ -1307,7 +1317,7 @@ int   size;
 	fprintf (file, "\t*%s--\n", sp);
       else
 	{
-	  fprintf (file, "\t%s=%d\n\t*%s++%s\n", 
+	  fprintf (file, "\t%s=%ld\n\t*%s++%s\n", 
 		   reg_names[REG_J], -current_frame_info.args_size, sp, reg_names[REG_J]);
 	}
     }
@@ -1354,7 +1364,7 @@ int   size;
 	fprintf (file, "\t*%s--\n", sp);
       else
 	{
-	  fprintf (file, "\t%s=%d\n\t*%s++%s\n", 
+	  fprintf (file, "\t%s=%ld\n\t*%s++%s\n", 
 		   reg_names[REG_J], -current_frame_info.var_size, sp, reg_names[REG_J]);
 	}
     }
@@ -1389,7 +1399,7 @@ emit_move_sequence (operands, mode)
 
 void
 double_reg_from_memory (operands)
-rtx operands[];
+     rtx operands[];
 {
     rtx xoperands[4];
 
@@ -1410,7 +1420,6 @@ rtx operands[];
     else if (GET_CODE(XEXP(operands[1],0)) == PLUS)
     {
       rtx addr;
-      rtx base;
       int offset;
 
       output_asm_insn ("%u0=%1", operands);
@@ -1438,7 +1447,7 @@ rtx operands[];
 
 void
 double_reg_to_memory (operands)
-rtx operands[];
+     rtx operands[];
 {
     rtx xoperands[4];
 
@@ -1489,6 +1498,8 @@ rtx operands[];
 void
 override_options ()
 {
+  char *tmp;
+
   if (chip_name == (char *) 0)
     chip_name = DEFAULT_CHIP_NAME;
 
@@ -1504,22 +1515,23 @@ override_options ()
   if (const_seg_name == (char *) 0)
     const_seg_name = DEFAULT_CONST_SEG_NAME;
   
-  save_chip_name = (char *) xmalloc (strlen(chip_name) + 1);
-  strcpy (save_chip_name, chip_name);
+  save_chip_name = xstrdup (chip_name);
 
-  rsect_text = (char *) xmalloc (strlen(".rsect ") + 
-				 strlen(text_seg_name) + 3);
-  rsect_data = (char *) xmalloc (strlen(".rsect ") + 
-				 strlen(data_seg_name) + 3);
-  rsect_bss = (char *) xmalloc (strlen(".rsect ") + 
-				strlen(bss_seg_name) + 3);
-  rsect_const = (char *) xmalloc (strlen(".rsect ") + 
-				  strlen(const_seg_name) + 3);
-  
-  sprintf (rsect_text, ".rsect \"%s\"", text_seg_name);
-  sprintf (rsect_data, ".rsect \"%s\"", data_seg_name);
-  sprintf (rsect_bss,  ".rsect \"%s\"", bss_seg_name);
-  sprintf (rsect_const, ".rsect \"%s\"", const_seg_name);
+  rsect_text = tmp = (char *) xmalloc (strlen(".rsect ") + 
+				       strlen(text_seg_name) + 3);
+  sprintf (tmp, ".rsect \"%s\"", text_seg_name);
+
+  rsect_data = tmp = (char *) xmalloc (strlen(".rsect ") + 
+				       strlen(data_seg_name) + 3);
+  sprintf (tmp, ".rsect \"%s\"", data_seg_name);
+
+  rsect_bss = tmp = (char *) xmalloc (strlen(".rsect ") + 
+				      strlen(bss_seg_name) + 3);
+  sprintf (tmp,  ".rsect \"%s\"", bss_seg_name);
+
+  rsect_const = tmp = (char *) xmalloc (strlen(".rsect ") + 
+					strlen(const_seg_name) + 3);
+  sprintf (tmp, ".rsect \"%s\"", const_seg_name);
   
   if (optimize)
     {
@@ -1529,11 +1541,35 @@ override_options ()
 	  flag_inline_functions = 1;
 	}
     }
+
+  /* Mark our global variables for GC.  */
+  ggc_add_rtx_root (&dsp16xx_addhf3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_subhf3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_mulhf3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_divhf3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_cmphf3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_fixhfhi2_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_floathihf2_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_neghf2_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_mulhi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_udivqi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_udivhi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_divqi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_divhi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_modqi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_modhi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_umodqi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_umodhi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_ashrhi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_ashlhi3_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_ucmphi2_libcall, 1);
+  ggc_add_rtx_root (&dsp16xx_lshrhi3_libcall, 1);
 }
 
+
 enum rtx_code
 next_cc_user_code (insn)
-rtx insn;
+     rtx insn;
 {
   if ( !(insn = next_cc0_user (insn)))
     abort ();
@@ -1567,9 +1603,9 @@ next_cc_user_unsigned (insn)
 
 void
 print_operand(file, op, letter)
-FILE *file;
-rtx op;
-int letter;
+     FILE *file;
+     rtx op;
+     int letter;
 {
     enum rtx_code code;
 
@@ -1663,8 +1699,8 @@ int letter;
 
 void
 print_operand_address(file, addr)
-FILE *file;
-rtx addr;
+     FILE *file;
+     rtx addr;
 {
   rtx base;
   int offset;
@@ -1708,9 +1744,8 @@ rtx addr;
 
 void
 output_dsp16xx_float_const(operands)
-rtx *operands;
+     rtx *operands;
 {
-  rtx dst = operands[0];
   rtx src = operands[1];
   
 #if HOST_FLOAT_FORMAT == TARGET_FLOAT_FORMAT
@@ -1727,7 +1762,7 @@ rtx *operands;
 #endif
 }
 
-int
+static int
 reg_save_size ()
 {
   int reg_save_size = 0;
@@ -1760,8 +1795,6 @@ dsp16xx_starting_frame_offset()
 int
 initial_frame_pointer_offset()
 {
-  int frame_size;
-  int regno;
   int offset = 0;
   
   offset = compute_frame_size (get_frame_size());
@@ -1779,10 +1812,10 @@ initial_frame_pointer_offset()
 #if 0
 void
 emit_1600_core_shift (shift_op, operands, shift_amount, mode)
-enum rtx_code shift_op;
-rtx *operands;
-int shift_amount;
-enum machine_mode mode;
+     enum rtx_code shift_op;
+     rtx *operands;
+     int shift_amount;
+     enum machine_mode mode;
 {
   int quotient;
   int i;
@@ -1795,10 +1828,11 @@ enum machine_mode mode;
 	  quotient = shift_amount/16;
 	  shift_amount = shift_amount - (quotient * 16);
 	  for (i = 0; i < quotient; i++)
-	    emit_insn (gen_rtx (SET, VOIDmode, operands[0],
-				gen_rtx (shift_op, mode, 
-					 first_shift_emitted ? operands[0] : operands[1],
-					 GEN_INT (16))));
+	    emit_insn (gen_rtx_SET (VOIDmode, operands[0],
+				    gen_rtx (shift_op, mode, 
+					     first_shift_emitted
+					     ? operands[0] : operands[1],
+					     GEN_INT (16))));
 	  first_shift_emitted = 1;
 	}
       else if (shift_amount/8)
@@ -1806,10 +1840,11 @@ enum machine_mode mode;
 	  quotient = shift_amount/8;
 	  shift_amount = shift_amount - (quotient * 8);
 	  for (i = 0; i < quotient; i++)
-	    emit_insn (gen_rtx (SET, VOIDmode, operands[0],
-				gen_rtx (shift_op, mode, 
-					 first_shift_emitted ? operands[0] : operands[1],
-					 GEN_INT (8))));
+	    emit_insn (gen_rtx_SET (VOIDmode, operands[0],
+				    gen_rtx (shift_op, mode, 
+					     first_shift_emitted
+					     ? operands[0] : operands[1],
+					     GEN_INT (8))));
 	  first_shift_emitted = 1;
 	}
       else if (shift_amount/4)
@@ -1817,10 +1852,11 @@ enum machine_mode mode;
 	  quotient = shift_amount/4;
 	  shift_amount = shift_amount - (quotient * 4);
 	  for (i = 0; i < quotient; i++)
-	    emit_insn (gen_rtx (SET, VOIDmode, operands[0],
-				gen_rtx (shift_op, mode, 
-					 first_shift_emitted ? operands[0] : operands[1],
-					 GEN_INT (4))));
+	    emit_insn (gen_rtx_SET (VOIDmode, operands[0],
+				    gen_rtx (shift_op, mode, 
+					     first_shift_emitted
+					     ? operands[0] : operands[1],
+					     GEN_INT (4))));
 	  first_shift_emitted = 1;
 	}
       else if (shift_amount/1)
@@ -1828,10 +1864,11 @@ enum machine_mode mode;
 	  quotient = shift_amount/1;
 	  shift_amount = shift_amount - (quotient * 1);
 	  for (i = 0; i < quotient; i++)
-	    emit_insn (gen_rtx (SET, VOIDmode, operands[0],
-				gen_rtx (shift_op, mode, 
-					 first_shift_emitted ? operands[0] : operands[1],
-					 GEN_INT (1))));
+	    emit_insn (gen_rtx_SET (VOIDmode, operands[0],
+				    gen_rtx (shift_op, mode, 
+					     first_shift_emitted
+					     ? operands[0] : operands[1],
+					     GEN_INT (1))));
 	  first_shift_emitted = 1;
 	}
     }
@@ -1839,15 +1876,15 @@ enum machine_mode mode;
 #else
 void
 emit_1600_core_shift (shift_op, operands, shift_amount)
-enum rtx_code shift_op;
-rtx *operands;
-int shift_amount;
+     enum rtx_code shift_op;
+     rtx *operands;
+     int shift_amount;
 {
   int quotient;
   int i;
   int first_shift_emitted = 0;
-  char **shift_asm_ptr;
-  char **shift_asm_ptr_first;
+  const char * const *shift_asm_ptr;
+  const char * const *shift_asm_ptr_first;
 
   if (shift_op == ASHIFT)
     {
@@ -1909,11 +1946,11 @@ int shift_amount;
 }
 #endif
 void
-  asm_output_common(file, name, size, rounded)
-FILE *file;
-char *name;
-int size;
-int rounded;
+asm_output_common(file, name, size, rounded)
+     FILE *file;
+     const char *name;
+     int size ATTRIBUTE_UNUSED;
+     int rounded;
 {
     bss_section ();
     ASM_GLOBALIZE_LABEL (file, name);
@@ -1927,10 +1964,10 @@ int rounded;
 
 void
 asm_output_local(file, name, size, rounded)
-FILE *file;
-char *name;
-int size;
-int rounded;
+     FILE *file;
+     const char *name;
+     int size ATTRIBUTE_UNUSED;
+     int rounded;
 {
     bss_section ();
     assemble_name (file, name);
@@ -1943,8 +1980,8 @@ int rounded;
 
 void
 asm_output_float (file, fp_const)
-FILE *file;
-double fp_const;
+     FILE *file;
+     double fp_const;
 {
 #if HOST_FLOAT_FORMAT == TARGET_FLOAT_FORMAT
       REAL_VALUE_TYPE d = fp_const;
@@ -1953,9 +1990,9 @@ double fp_const;
       REAL_VALUE_TO_TARGET_SINGLE (d, value);
       fputs ("\tint ", file);
 #ifdef WORDS_BIG_ENDIAN
-      fprintf (file, "0x%-4.4x, 0x%-4.4x", (value >> 16) & 0xffff, (value & 0xffff));
+      fprintf (file, "0x%-4.4lx, 0x%-4.4lx", (value >> 16) & 0xffff, (value & 0xffff));
 #else
-      fprintf (file, "0x%-4.4x, 0x%-4.4x", (value & 0xffff), (value >> 16) & 0xffff);
+      fprintf (file, "0x%-4.4lx, 0x%-4.4lx", (value & 0xffff), (value >> 16) & 0xffff);
 #endif
       fputs ("\n", file);
 #else
@@ -1965,21 +2002,21 @@ double fp_const;
 
 void
 asm_output_long (file, value)
-FILE *file;
-long value;
+     FILE *file;
+     long value;
 {
       fputs ("\tint ", file);
 #ifdef WORDS_BIG_ENDIAN
-      fprintf (file, "0x%-4.4x, 0x%-4.4x", (value >> 16) & 0xffff, (value & 0xffff));
+      fprintf (file, "0x%-4.4lx, 0x%-4.4lx", (value >> 16) & 0xffff, (value & 0xffff));
 #else
-      fprintf (file, "0x%-4.4x, 0x%-4.4x", (value & 0xffff), (value >> 16) & 0xffff);
+      fprintf (file, "0x%-4.4lx, 0x%-4.4lx", (value & 0xffff), (value >> 16) & 0xffff);
 #endif
       fputs ("\n", file);
 }
 
 int
 dsp16xx_address_cost (addr)
-rtx addr;
+     rtx addr;
 {
     switch (GET_CODE (addr))
     {
@@ -2085,7 +2122,7 @@ dsp16xx_function_arg (args_so_far, mode, type, named)
 	args_so_far++;
 
       if (named && args_so_far < 4 && !MUST_PASS_IN_STACK (mode,type))
-	return gen_rtx (REG, mode, args_so_far + FIRST_REG_FOR_FUNCTION_ARG);
+	return gen_rtx_REG (mode, args_so_far + FIRST_REG_FOR_FUNCTION_ARG);
       else
 	return (struct rtx_def *) 0;
     }
@@ -2100,7 +2137,7 @@ dsp16xx_function_arg_advance (cum, mode, type, named)
      CUMULATIVE_ARGS *cum;	/* current arg information */
      enum machine_mode mode;	/* current arg mode */
      tree type;			/* type of the argument or 0 if lib support */
-     int named;			/* whether or not the argument was named */
+     int named ATTRIBUTE_UNUSED;/* whether or not the argument was named */
 {
   if (TARGET_REGPARM)
     {
@@ -2135,14 +2172,14 @@ gen_tst_reg (x)
 
   if (mode == QImode)
     {
-	  emit_insn (gen_rtx (PARALLEL, VOIDmode,
-			gen_rtvec (2,
-				   gen_rtx (SET, VOIDmode, cc0_rtx, x),
-				   gen_rtx (CLOBBER, VOIDmode,
-					    gen_rtx (SCRATCH, QImode, 0)))));
+	  emit_insn (gen_rtx_PARALLEL
+		     (VOIDmode,
+		      gen_rtvec (2, gen_rtx_SET (VOIDmode, cc0_rtx, x),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)))));
 	}
   else if (mode == HImode)
-    emit_insn (gen_rtx (SET, VOIDmode, cc0_rtx, x));
+    emit_insn (gen_rtx_SET (VOIDmode, cc0_rtx, x));
   else
     fatal ("Invalid mode for gen_tst_reg");
 
@@ -2165,54 +2202,64 @@ gen_compare_reg (code, x, y)
 
   if (mode == QImode)
     {
-      if (code == GTU || code == GEU ||
-	  code == LTU || code == LEU)
+      if (code == GTU || code == GEU
+	  || code == LTU || code == LEU)
 	{
-	  emit_insn (gen_rtx (PARALLEL, VOIDmode,
-			gen_rtvec (3,
-				   gen_rtx (SET, VOIDmode, cc0_rtx,
-					    gen_rtx (COMPARE, mode, x, y)),
-				   gen_rtx (CLOBBER, VOIDmode,
-					    gen_rtx (SCRATCH, QImode, 0)),
-				   gen_rtx (CLOBBER, VOIDmode,
-					    gen_rtx (SCRATCH, QImode, 0)))));
+	  emit_insn (gen_rtx_PARALLEL
+		     (VOIDmode,
+		      gen_rtvec (3,
+				 gen_rtx_SET (VOIDmode, cc0_rtx,
+					      gen_rtx_COMPARE (mode, x, y)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)))));
 	}
       else
 	{
-	  emit_insn (gen_rtx (PARALLEL, VOIDmode,
-			gen_rtvec (3,
-				   gen_rtx (SET, VOIDmode, cc0_rtx,
-					    gen_rtx (COMPARE, mode, x, y)),
-				   gen_rtx (CLOBBER, VOIDmode,
-					    gen_rtx (SCRATCH, QImode, 0)),
-				   gen_rtx (CLOBBER, VOIDmode,
-					    gen_rtx (SCRATCH, QImode, 0)))));
+	  emit_insn (gen_rtx_PARALLEL
+		     (VOIDmode,
+		      gen_rtvec (3, gen_rtx_SET (VOIDmode, cc0_rtx,
+						 gen_rtx_COMPARE (mode, x, y)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)))));
 	}
     }
   else if (mode == HImode)
     {
-      if (code == GTU || code == GEU ||
-	  code == LTU || code == LEU)
+      if (code == GTU || code == GEU
+	  || code == LTU || code == LEU)
 	{
 #if 1
-	  emit_insn (gen_rtx (PARALLEL, VOIDmode, gen_rtvec (5,
-			     gen_rtx (SET, VOIDmode, cc0_rtx, gen_rtx (COMPARE, VOIDmode, x, y)),
-		             gen_rtx (CLOBBER, VOIDmode, gen_rtx (SCRATCH, QImode, 0)),
-		             gen_rtx (CLOBBER, VOIDmode, gen_rtx (SCRATCH, QImode, 0)),
-		             gen_rtx (CLOBBER, VOIDmode, gen_rtx (SCRATCH, QImode, 0)),
-		             gen_rtx (CLOBBER, VOIDmode, gen_rtx (SCRATCH, QImode, 0)))));
+	  emit_insn (gen_rtx_PARALLEL
+		     (VOIDmode,
+		      gen_rtvec (5,
+				 gen_rtx_SET (VOIDmode, cc0_rtx,
+					      gen_rtx_COMPARE (VOIDmode,
+							       x, y)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)),
+				 gen_rtx_CLOBBER (VOIDmode,
+						  gen_rtx_SCRATCH (QImode)))));
 #else
 	  if (!dsp16xx_ucmphi2_libcall)
-	    dsp16xx_ucmphi2_libcall = gen_rtx (SYMBOL_REF, Pmode, UCMPHI2_LIBCALL);
+	    dsp16xx_ucmphi2_libcall = gen_rtx_SYMBOL_REF (Pmode, UCMPHI2_LIBCALL);
 	  emit_library_call (dsp16xx_ucmphi2_libcall, 1, HImode, 2,
 			     x, HImode, y, HImode);
 	  emit_insn (gen_tsthi_1 (copy_to_reg(hard_libcall_value (HImode))));
 #endif
 	}
       else
-	emit_insn (gen_rtx (SET, VOIDmode, cc0_rtx,
-			    gen_rtx (COMPARE, VOIDmode, force_reg(HImode, x), 
-				     force_reg(HImode,y))));
+	emit_insn (gen_rtx_SET (VOIDmode, cc0_rtx,
+				gen_rtx_COMPARE (VOIDmode,
+						 force_reg (HImode, x), 
+						 force_reg (HImode,y))));
     }
   else
     fatal ("Invalid mode for integer comparison in gen_compare_reg");
@@ -2220,7 +2267,7 @@ gen_compare_reg (code, x, y)
   return cc0_rtx;
 }
 
-char *
+const char *
 output_block_move (operands)
      rtx operands[];
 {
