@@ -2,7 +2,8 @@
    hosting on Windows NT 3.x, using a Unix style C library and tools,
    as distinct from winnt.h, which is used to build GCC for use with a
    windows style library and tool set and uses the Microsoft tools.
-   Copyright (C) 1995, 1996, 1997, 1998, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000
+   Free Software Foundation, Inc.
 
 This file is part of GNU CC.
 
@@ -45,13 +46,18 @@ Boston, MA 02111-1307, USA. */
 
 #undef  SUBTARGET_SWITCHES
 #define SUBTARGET_SWITCHES \
-{ "cygwin",		  MASK_CYGWIN, "Use the Cygwin interface" },  \
-{ "no-cygwin",		  MASK_WIN32, "Use the Mingw32 interface" }, \
-{ "windows",		  MASK_WINDOWS, "Create GUI application" }, \
-{ "console",		  -MASK_WINDOWS, "Create console application" }, \
-{ "dll",		  MASK_DLL, "Generate code for a DLL" },     \
-{ "nop-fun-dllimport",	  MASK_NOP_FUN_DLLIMPORT, "Ignore dllimport for functions" }, \
-{ "no-nop-fun-dllimport", -MASK_NOP_FUN_DLLIMPORT, "" },
+{ "cygwin",		  MASK_CYGWIN,					\
+  N_("Use the Cygwin interface") },					\
+{ "no-cygwin",		  MASK_WIN32,					\
+  N_("Use the Mingw32 interface") },					\
+{ "windows",		  MASK_WINDOWS, N_("Create GUI application") },	\
+{ "console",		  -MASK_WINDOWS,				\
+  N_("Create console application") }, 					\
+{ "dll",		  MASK_DLL, N_("Generate code for a DLL") },	\
+{ "nop-fun-dllimport",	  MASK_NOP_FUN_DLLIMPORT,			\
+  N_("Ignore dllimport for functions") }, 				\
+{ "no-nop-fun-dllimport", -MASK_NOP_FUN_DLLIMPORT, "" }, \
+{ "threads",		  0, N_("Use Mingw-specific thread support") },
 
 
 /* Support the __declspec keyword by turning them into attributes.
@@ -62,16 +68,10 @@ Boston, MA 02111-1307, USA. */
    is that args are not accumulated: each new appearance would clobber any
    existing args.  */
 
-#ifdef CPP_PREDEFINES
 #undef CPP_PREDEFINES
-#endif
-
-#define CPP_PREDEFINES "-Di386 -D_WIN32 \
-  -DWINNT  -D_X86_=1 -D__STDC__=1\
-  -D__stdcall=__attribute__((__stdcall__)) \
-  -D__cdecl=__attribute__((__cdecl__)) \
-  -D__declspec(x)=__attribute__((x)) \
-  -Asystem(winnt) -Acpu(i386) -Amachine(i386)"
+#define CPP_PREDEFINES "-D_WIN32 \
+  -DWINNT  -D_X86_=1 \
+  -Asystem(winnt)"
 
 /* Normally, -lgcc is not needed since everything in it is in the DLL, but we
    want to allow things to be added to it when installing new versions of
@@ -85,6 +85,11 @@ Boston, MA 02111-1307, USA. */
 
 #undef CPP_SPEC
 #define CPP_SPEC "-remap %(cpp_cpu) %{posix:-D_POSIX_SOURCE} \
+  -D__stdcall=__attribute__((__stdcall__)) \
+  -D__cdecl=__attribute__((__cdecl__)) \
+  %{!ansi:-D_stdcall=__attribute__((__stdcall__)) \
+    -D_cdecl=__attribute__((__cdecl__))} \
+  -D__declspec(x)=__attribute__((x)) \
   %{!mno-cygwin:-D__CYGWIN32__ -D__CYGWIN__} \
   %{mno-cygwin:-iwithprefixbefore \
     ../../../../%(mingw_include_path)/include/mingw32 -D__MINGW32__=0.2}"
@@ -127,8 +132,6 @@ Boston, MA 02111-1307, USA. */
 #define WCHAR_TYPE_SIZE 16
 #define WCHAR_TYPE "short unsigned int"
 
-#define HAVE_ATEXIT 1
-
 
 /* Enable parsing of #pragma pack(push,<n>) and #pragma pack(pop).  */
 #define HANDLE_PRAGMA_PACK_PUSH_POP 1
@@ -299,16 +302,17 @@ extern void i386_pe_encode_section_info ();
 #undef  STRIP_NAME_ENCODING
 #define STRIP_NAME_ENCODING(VAR,SYMBOL_NAME)				\
 do {									\
-  char *_p;								\
-  char *_name = I386_PE_STRIP_ENCODING (SYMBOL_NAME);			\
+  const char *_p;							\
+  const char *_name = I386_PE_STRIP_ENCODING (SYMBOL_NAME);		\
   for (_p = _name; *_p && *_p != '@'; ++_p)				\
     ;									\
   if (*_p == '@')							\
     {									\
       int _len = _p - _name;						\
-      (VAR) = (char *) alloca (_len + 1);				\
-      strncpy ((VAR), _name, _len);					\
-      (VAR)[_len] = '\0';						\
+      char *_new_name = (char *) alloca (_len + 1);			\
+      strncpy (_new_name, _name, _len);					\
+      _new_name[_len] = '\0';						\
+      (VAR) = _new_name;						\
     }									\
   else									\
     (VAR) = _name;							\
@@ -326,7 +330,7 @@ do {									\
 #define ASM_OUTPUT_COMMON(STREAM, NAME, SIZE, ROUNDED)	\
 do {							\
   if (i386_pe_dllexport_name_p (NAME))			\
-    i386_pe_record_exported_symbol (NAME);		\
+    i386_pe_record_exported_symbol (NAME, 1);		\
   if (! i386_pe_dllimport_name_p (NAME))		\
     {							\
       fprintf ((STREAM), "\t.comm\t"); 			\
@@ -341,7 +345,7 @@ do {							\
 #define ASM_DECLARE_OBJECT_NAME(STREAM, NAME, DECL) 	\
 do {							\
   if (i386_pe_dllexport_name_p (NAME))			\
-    i386_pe_record_exported_symbol (NAME);		\
+    i386_pe_record_exported_symbol (NAME, 1);		\
   ASM_OUTPUT_LABEL ((STREAM), (NAME));			\
 } while (0)
 
@@ -392,7 +396,7 @@ do {									\
       enum sect_enum {SECT_RW, SECT_RO, SECT_EXEC} type;		\
     } *sections;							\
   struct section_info *s;						\
-  char *mode;								\
+  const char *mode;							\
   enum sect_enum type;							\
 									\
   for (s = sections; s; s = s->next)					\
@@ -404,7 +408,14 @@ do {									\
   else if (DECL && DECL_READONLY_SECTION (DECL, RELOC))			\
     type = SECT_RO, mode = "";						\
   else									\
-    type = SECT_RW, mode = "w";						\
+    {									\
+      type = SECT_RW;							\
+      if (TREE_CODE (DECL) == VAR_DECL					\
+          && lookup_attribute ("shared", DECL_MACHINE_ATTRIBUTES (DECL))) \
+        mode = "ws";							\
+      else								\
+        mode = "w";							\
+    }									\
 									\
   if (s == 0)								\
     {									\
@@ -437,7 +448,7 @@ do {									\
   do									\
     {									\
       if (i386_pe_dllexport_name_p (NAME))				\
-	i386_pe_record_exported_symbol (NAME);				\
+	i386_pe_record_exported_symbol (NAME, 0);			\
       if (write_symbols != SDB_DEBUG)					\
 	i386_pe_declare_function_type (FILE, NAME, TREE_PUBLIC (DECL));	\
       ASM_OUTPUT_LABEL (FILE, NAME);					\
@@ -458,6 +469,11 @@ do {									\
 #define ASM_OUTPUT_EXTERNAL_LIBCALL(FILE, FUN) \
   i386_pe_declare_function_type (FILE, XSTR (FUN, 0), 1)
 
+/* This says out to put a global symbol in the BSS section. */
+#undef ASM_OUTPUT_ALIGNED_BSS
+#define ASM_OUTPUT_ALIGNED_BSS(FILE, DECL, NAME, SIZE, ALIGN) \
+  asm_output_aligned_bss ((FILE), (DECL), (NAME), (SIZE), (ALIGN))
+
 /* Output function declarations at the end of the file.  */
 #define ASM_FILE_END(FILE) \
   i386_pe_asm_file_end (FILE)
@@ -476,35 +492,32 @@ do {									\
       && strcmp (IDENTIFIER_POINTER (DECL_NAME (current_function_decl)),\
 		 "main") == 0)						\
      {									\
-      rtx xops[1];							\
-      xops[0] = gen_rtx_MEM (FUNCTION_MODE,				\
-			 gen_rtx (SYMBOL_REF, Pmode, "_monstartup"));	\
-      if (do_rtl)							\
-	emit_call_insn (gen_rtx (CALL, VOIDmode, xops[0], const0_rtx));	\
-      else								\
-	output_asm_insn (AS1 (call,%P1), xops);			\
+      emit_call_insn (gen_rtx (CALL, VOIDmode, 				\
+        gen_rtx_MEM (FUNCTION_MODE,					\
+		     gen_rtx_SYMBOL_REF (Pmode, "_monstartup")),	\
+	const0_rtx));							\
      }
 
 /* External function declarations.  */
 
-#ifndef PROTO
+#ifndef PARAMS
 #if defined (USE_PROTOTYPES) ? USE_PROTOTYPES : defined (__STDC__)
-#define PROTO(ARGS) ARGS
+#define PARAMS(ARGS) ARGS
 #else
-#define PROTO(ARGS) ()
+#define PARAMS(ARGS) ()
 #endif
 #endif
 
 #ifdef BUFSIZ		/* stdio.h has been included, ok to use FILE * */
-#define STDIO_PROTO(ARGS) PROTO(ARGS)
+#define STDIO_PARAMS(ARGS) PARAMS(ARGS)
 #else
-#define STDIO_PROTO(ARGS) ()
+#define STDIO_PARAMS(ARGS) ()
 #endif
 
-extern void i386_pe_record_external_function PROTO((char *));
-extern void i386_pe_declare_function_type STDIO_PROTO((FILE *, char *, int));
-extern void i386_pe_record_exported_symbol PROTO((char *));
-extern void i386_pe_asm_file_end STDIO_PROTO((FILE *));
+extern void i386_pe_record_external_function PARAMS ((char *));
+extern void i386_pe_declare_function_type STDIO_PARAMS ((FILE *, char *, int));
+extern void i386_pe_record_exported_symbol PARAMS ((char *, int));
+extern void i386_pe_asm_file_end STDIO_PARAMS ((FILE *));
 
 /* For Win32 ABI compatibility */
 #undef DEFAULT_PCC_STRUCT_RETURN
@@ -523,3 +536,7 @@ extern void i386_pe_asm_file_end STDIO_PROTO((FILE *));
 #define SET_ASM_OP "\t.set"
 #endif
 
+#ifndef INT_ASM_OP
+#define INT_ASM_OP ".long"
+#endif
+
