@@ -1,6 +1,6 @@
 /* Definitions of target machine for GNU compiler.
    AT&T UNIX PC version (pc7300, 3b1)
-   Copyright (C) 1987, 1993, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1987, 1993, 1996, 1999 Free Software Foundation, Inc.
    Contributed by Alex Crain (alex@umbc3.umd.edu).
 
 This file is part of GNU CC.
@@ -99,7 +99,7 @@ output_file_directive ((FILE), main_input_filename)
 
 /* The 3b1 does not have `atexit'.  */
 
-#undef HAVE_ATEXIT
+#define NEED_ATEXIT
 
 /* Override parts of m68k.h to fit the SGS-3b1 assembler.  */
 
@@ -508,6 +508,5 @@ do { fprintf (asm_out_file, "\ttag\t");	\
 /* Definitions for collect2.  */
 
 #define OBJECT_FORMAT_COFF
-#define NO_SYS_SIGLIST
 #define MY_ISCOFF(magic) \
  ((magic) == MC68KWRMAGIC || (magic) == MC68KROMAGIC || (magic) == MC68KPGMAGIC)
