@@ -1,5 +1,6 @@
 /* Definitions of target machine for GNU compiler.  SONY NEWS-OS 4 version.
-   Copyright (C) 1987, 89, 93, 94, 96, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1987, 1989, 1993, 1994, 1996, 1997, 1998, 1999
+   Free Software Foundation, Inc.
 
 This file is part of GNU CC.
 
@@ -126,11 +127,12 @@ Boston, MA 02111-1307, USA.  */
 
 #define FUNCTION_VALUE(VALTYPE,FUNC) LIBCALL_VALUE (TYPE_MODE (VALTYPE))
 
-#define LIBCALL_VALUE(MODE)						   \
- gen_rtx (REG, (MODE),							   \
-	  ((TARGET_68881						   \
-	    && ((MODE) == SFmode || (MODE) == DFmode || (MODE) == XFmode)) \
-	   ? 16 : 0))
+#define LIBCALL_VALUE(MODE)					\
+ gen_rtx_REG ((MODE),						\
+	      ((TARGET_68881					\
+		&& ((MODE) == SFmode || (MODE) == DFmode	\
+		    || (MODE) == XFmode))			\
+	       ? 16 : 0))
 
 #define ASM_OUTPUT_ALIGN(FILE,LOG)	\
   fprintf (FILE, "\t.align %d\n", (LOG))
