@@ -1,5 +1,6 @@
 /* Subroutines used for code generation on ROMP.
-   Copyright (C) 1990, 1991, 1992, 1993, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1990, 1991, 1992, 1993, 1997, 1998, 1999, 2000
+   Free Software Foundation, Inc.
    Contributed by Richard Kenner (kenner@nyu.edu)
 
 This file is part of GNU CC.
@@ -21,7 +22,7 @@ Boston, MA 02111-1307, USA.  */
 
 
 #include "config.h"
-#include <stdio.h>
+#include "system.h"
 #include "rtl.h"
 #include "regs.h"
 #include "hard-reg-set.h"
@@ -36,14 +37,18 @@ Boston, MA 02111-1307, USA.  */
 #include "expr.h"
 #include "obstack.h"
 #include "tree.h"
+#include "function.h"
+#include "tm_p.h"
 
 #define min(A,B)	((A) < (B) ? (A) : (B))
 #define max(A,B)	((A) > (B) ? (A) : (B))
 
-static int unsigned_comparisons_p ();
-static void output_loadsave_fpregs ();
-static void output_fpops ();
-static void init_fpops ();
+static int unsigned_comparisons_p PARAMS ((rtx));
+static void output_loadsave_fpregs PARAMS ((FILE *, enum rtx_code, rtx));
+static void output_fpops PARAMS ((FILE *));
+static void init_fpops PARAMS ((void));
+static int memory_offset_in_range_p PARAMS ((rtx, enum machine_mode, int, int));
+static unsigned int hash_rtx PARAMS ((rtx));
 
 /* Return 1 if the insn using CC0 set by INSN does not contain
    any unsigned tests applied to the condition codes.
@@ -74,7 +79,7 @@ static int
 unsigned_comparisons_p (x)
      rtx x;
 {
-  register char *fmt;
+  register const char *fmt;
   register int len, i;
   register enum rtx_code code = GET_CODE (x);
 
@@ -95,6 +100,8 @@ unsigned_comparisons_p (x)
     case LEU:
     case GEU:
       return (XEXP (x, 0) == cc0_rtx || XEXP (x, 1) == cc0_rtx);
+    default:
+      break;
     }
 
   len = GET_RTX_LENGTH (code);
@@ -125,8 +132,9 @@ unsigned_comparisons_p (x)
    cc_state.value[12] refer to two possible values that might correspond
    to the CC.  We only store register values.  */
 
+void
 update_cc (body, insn)
-    rtx body;
+    rtx body ATTRIBUTE_UNUSED;
     rtx insn;
 {
   switch (get_attr_cc (insn))
@@ -139,11 +147,11 @@ update_cc (body, insn)
       /* Insn doesn't affect the CC but does modify operand[0], known to be
 	 a register.  */
       if (cc_status.value1 != 0
-	  && reg_overlap_mentioned_p (recog_operand[0], cc_status.value1))
+	  && reg_overlap_mentioned_p (recog_data.operand[0], cc_status.value1))
 	cc_status.value1 = 0;
 
       if (cc_status.value2 != 0
-	  && reg_overlap_mentioned_p (recog_operand[0], cc_status.value2))
+	  && reg_overlap_mentioned_p (recog_data.operand[0], cc_status.value2))
 	cc_status.value2 = 0;
 
       break;
@@ -152,20 +160,20 @@ update_cc (body, insn)
       /* Insn copies operand[1] to operand[0], both registers, but doesn't
          affect the CC.  */
       if (cc_status.value1 != 0
-	  && reg_overlap_mentioned_p (recog_operand[0], cc_status.value1))
+	  && reg_overlap_mentioned_p (recog_data.operand[0], cc_status.value1))
 	cc_status.value1 = 0;
 
       if (cc_status.value2 != 0
-	  && reg_overlap_mentioned_p (recog_operand[0], cc_status.value2))
+	  && reg_overlap_mentioned_p (recog_data.operand[0], cc_status.value2))
 	cc_status.value2 = 0;
 
       if (cc_status.value1 != 0
-	  && rtx_equal_p (cc_status.value1, recog_operand[1]))
-	cc_status.value2 = recog_operand[0];
+	  && rtx_equal_p (cc_status.value1, recog_data.operand[1]))
+	cc_status.value2 = recog_data.operand[0];
 
       if (cc_status.value2 != 0
-	  && rtx_equal_p (cc_status.value2, recog_operand[1]))
-	cc_status.value1 = recog_operand[0];
+	  && rtx_equal_p (cc_status.value2, recog_data.operand[1]))
+	cc_status.value1 = recog_data.operand[0];
 
       break;
 
@@ -175,10 +183,10 @@ update_cc (body, insn)
       break;
 
     case CC_SETS:
-      /* Insn sets CC to recog_operand[0], but overflow is impossible.  */
+      /* Insn sets CC to recog_data.operand[0], but overflow is impossible.  */
       CC_STATUS_INIT;
       cc_status.flags |= CC_NO_OVERFLOW;
-      cc_status.value1 = recog_operand[0];
+      cc_status.value1 = recog_data.operand[0];
       break;
 
    case CC_COMPARE:
@@ -230,7 +238,7 @@ restore_compare_p (op)
 /*  Generate the (long) string corresponding to an inline multiply insn.
     Note that `r10' does not refer to the register r10, but rather to the
     SCR used as the MQ.  */
-char *
+const char *
 output_in_line_mul ()
 {
   static char insns[200];
@@ -355,7 +363,7 @@ symbolic_memory_operand (op, mode)
 int
 current_function_operand (op, mode)
      rtx op;
-     enum machine_mode mode;
+     enum machine_mode mode ATTRIBUTE_UNUSED;
 {
   if (GET_CODE (op) != MEM || GET_CODE (XEXP (op, 0)) != SYMBOL_REF
       ||  ! CONSTANT_POOL_ADDRESS_P (XEXP (op, 0)))
@@ -381,7 +389,7 @@ null_epilogue ()
 int
 constant_pool_address_operand (op, mode)
      rtx op;
-     enum machine_mode mode;
+     enum machine_mode mode ATTRIBUTE_UNUSED;
 {
   return ((GET_CODE (op) == SYMBOL_REF && CONSTANT_POOL_ADDRESS_P (op))
 	  || (GET_CODE (op) == CONST && GET_CODE (XEXP (op, 0)) == PLUS
@@ -396,7 +404,7 @@ constant_pool_address_operand (op, mode)
 int
 romp_symbolic_operand (op, mode)
      register rtx op;
-     enum machine_mode mode;
+     enum machine_mode mode ATTRIBUTE_UNUSED;
 {
   switch (GET_CODE (op))
     {
@@ -655,7 +663,7 @@ void
 print_operand (file, x, code)
     FILE *file;
     rtx x;
-    char code;
+    int code;
 {
   int i;
 
@@ -824,8 +832,8 @@ print_operand (file, x, code)
       if (GET_CODE (x) == REG)
 	fprintf (file, "%s", reg_names[REGNO (x) + 1]);
       else if (GET_CODE (x) == MEM)
-	print_operand (file, gen_rtx (MEM, GET_MODE (x),
-				      plus_constant (XEXP (x, 0), 4)), 0);
+	print_operand (file, gen_rtx_MEM (GET_MODE (x),
+					  plus_constant (XEXP (x, 0), 4)), 0);
       else
 	abort ();
       break;
@@ -1221,8 +1229,7 @@ output_epilog (file, size)
   /* Restore floating-point registers.  */
   if (write_code)
     output_loadsave_fpregs (file, CLOBBER,
-			    gen_rtx (PLUS, Pmode, gen_rtx (REG, Pmode, 1),
-				     GEN_INT (fp_save)));
+			    plus_constant (gen_rtx_REG (Pmode, 1), fp_save));
 
   /* If we push the stack and do not have size > 32K, adjust the register
      save location to the current position of sp.  Otherwise, if long frame,
@@ -1322,10 +1329,10 @@ static struct symref_hashent *symref_hash_table[SYMHASHSIZE];
    the name string is allocated from the permanent obstack.  */
 rtx
 get_symref (name)
-     register char *name;
+     register const char *name;
 {
   extern struct obstack permanent_obstack;
-  register char *sp = name;
+  register const char *sp = name;
   unsigned int hash = 0;
   struct symref_hashent *p, **last_p;
 
@@ -1348,9 +1355,9 @@ get_symref (name)
       end_temporary_allocation ();
       p = *last_p = (struct symref_hashent *)
 			permalloc (sizeof (struct symref_hashent));
-      p->symref = gen_rtx (SYMBOL_REF, Pmode,
-			   obstack_copy0 (&permanent_obstack,
-					  name, strlen (name)));
+      p->symref = gen_rtx_SYMBOL_REF (Pmode,
+				      obstack_copy0 (&permanent_obstack,
+						     name, strlen (name)));
       p->next = 0;
       resume_temporary_allocation ();
     }
@@ -1437,7 +1444,7 @@ hash_rtx (x)
   register unsigned int hash = (((int) GET_CODE (x) << 10)
 				+ ((int) GET_MODE (x) << 20));
   register int i;
-  register char *fmt = GET_RTX_FORMAT (GET_CODE (x));
+  register const char *fmt = GET_RTX_FORMAT (GET_CODE (x));
 
   for (i = 0; i < GET_RTX_LENGTH (GET_CODE (x)); i++)
     if (fmt[i] == 'e')
@@ -1459,11 +1466,11 @@ hash_rtx (x)
    A new floating-point operation block is created if this operation has not
    been seen before.  */
 
-char *
+const char *
 output_fpop (code, op0, op1, op2, insn)
      enum rtx_code code;
      rtx op0, op1, op2;
-     rtx insn;
+     rtx insn ATTRIBUTE_UNUSED;
 {
   static char outbuf[40];
   unsigned int hash, hash0, hash1, hash2;
@@ -1739,8 +1746,7 @@ output_loadsave_fpregs (file, code, addr)
 
   if (mask)
     fprintf (file, "\t%s\n",
-	     output_fpop (code, GEN_INT (mask),
-				gen_rtx (MEM, Pmode, addr),
+	     output_fpop (code, GEN_INT (mask), gen_rtx_MEM (Pmode, addr),
 				0, const0_rtx));
 
 }
@@ -1833,7 +1839,7 @@ output_fpops (file)
 	{
 	  register int type;
 	  register int opbyte;
-	  register char *desc0;
+	  register const char *desc0;
 	  char desc1[50];
 
 	  immed[i] = 0;
