@@ -20,8 +20,7 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 #include "config.h"
-#include <stdio.h>
-#include <ctype.h>
+#include "system.h"
 #include "tree.h"
 #include "rtl.h"
 #include "regs.h"
@@ -35,64 +34,26 @@ Boston, MA 02111-1307, USA.  */
 #include "flags.h"
 #include "recog.h"
 #include "expr.h"
+#include "function.h"
 #include "obstack.h"
 #include "toplev.h"
+#include "v850-protos.h"
 
 #ifndef streq
 #define streq(a,b) (strcmp (a, b) == 0)
 #endif
 
-/* Function prototypes that cannot exist in v850.h due to dependency
-   compilcations.  */
-extern rtx    function_arg
-  PROTO ((CUMULATIVE_ARGS *, enum machine_mode, tree, int));
-extern int    function_arg_partial_nregs
-  PROTO ((CUMULATIVE_ARGS *, enum machine_mode, tree, int));
-extern void   asm_file_start                PROTO ((FILE *));
-extern void   print_operand                 PROTO ((FILE *, rtx, int ));
-extern void   print_operand_address         PROTO ((FILE *, rtx));
-extern void   v850_output_aligned_bss
-  PROTO ((FILE *, tree, char *, int, int));
-extern void   v850_output_common
-  PROTO ((FILE *, tree, char *, int, int));
-extern void   v850_output_local
-  PROTO ((FILE *, tree, char *, int, int));
-extern int    const_costs                   PROTO ((rtx, enum rtx_code));
-extern char * output_move_double            PROTO ((rtx *));
-extern char * output_move_single            PROTO ((rtx *));
-extern int    ep_memory_operand
-  PROTO ((rtx, enum machine_mode, int));
-extern int    reg_or_0_operand              PROTO ((rtx, enum machine_mode));
-extern int    reg_or_int5_operand           PROTO ((rtx, enum machine_mode));
-extern int    call_address_operand          PROTO ((rtx, enum machine_mode));
-extern int    movsi_source_operand          PROTO ((rtx, enum machine_mode));
-extern int    power_of_two_operand          PROTO ((rtx, enum machine_mode));
-extern int    not_power_of_two_operand      PROTO ((rtx, enum machine_mode));
-extern int    special_symbolref_operand     PROTO ((rtx, enum machine_mode));
-extern void   v850_reorg                    PROTO ((rtx));
-extern void   notice_update_cc              PROTO ((rtx, rtx));
-extern int    v850_valid_machine_decl_attribute
-  PROTO ((tree, tree, tree));
-extern int    v850_interrupt_function_p     PROTO ((tree));
-extern int    pattern_is_ok_for_prologue    PROTO ((rtx, enum machine_mode));
-extern int    pattern_is_ok_for_epilogue    PROTO ((rtx, enum machine_mode));
-extern int    register_is_ok_for_epilogue   PROTO ((rtx, enum machine_mode));
-extern char * construct_save_jarl           PROTO ((rtx));
-extern char * construct_restore_jr          PROTO ((rtx));
-extern void   v850_encode_data_area         PROTO ((tree));
-extern void   v850_set_default_decl_attr    PROTO ((tree));
-
 /* Function prototypes for stupid compilers:  */
 static void const_double_split
-  PROTO ((rtx, HOST_WIDE_INT *, HOST_WIDE_INT *));
-static int  const_costs_int        PROTO ((HOST_WIDE_INT, int));
-static void substitute_ep_register PROTO ((rtx, rtx, int, int, rtx *, rtx *));
-static int  push_data_area         PROTO ((v850_data_area));
-static int  pop_data_area          PROTO ((v850_data_area));
-static int  parse_ghs_pragma_token PROTO ((char *));
-static int  ep_memory_offset       PROTO ((enum machine_mode, int));
-static int  mark_current_function_as_interrupt PROTO ((void));
-static void v850_set_data_area     PROTO ((tree, v850_data_area));
+  PARAMS ((rtx, HOST_WIDE_INT *, HOST_WIDE_INT *));
+static int  const_costs_int        PARAMS ((HOST_WIDE_INT, int));
+static void substitute_ep_register PARAMS ((rtx, rtx, int, int, rtx *, rtx *));
+static int  push_data_area         PARAMS ((v850_data_area));
+static int  pop_data_area          PARAMS ((v850_data_area));
+static int  parse_ghs_pragma_token PARAMS ((char *));
+static int  ep_memory_offset       PARAMS ((enum machine_mode, int));
+static int  mark_current_function_as_interrupt PARAMS ((void));
+static void v850_set_data_area     PARAMS ((tree, v850_data_area));
 
 /* True if the current function has anonymous arguments.  */
 int current_function_anonymous_args;
@@ -127,14 +88,14 @@ void
 override_options ()
 {
   int i;
-  extern int atoi PROTO ((const char *));
+  extern int atoi PARAMS ((const char *));
 
   /* Parse -m{s,t,z}da=nnn switches */
   for (i = 0; i < (int)SMALL_MEMORY_max; i++)
     {
       if (small_memory[i].value)
 	{
-	  if (!isdigit (*small_memory[i].value))
+	  if (!ISDIGIT (*small_memory[i].value))
 	    error ("%s=%s is not numeric.",
 		   small_memory[i].name,
 		   small_memory[i].value);
@@ -199,16 +160,16 @@ function_arg (cum, mode, type, named)
   switch (cum->nbytes / UNITS_PER_WORD)
     {
     case 0:
-      result = gen_rtx (REG, mode, 6);
+      result = gen_rtx_REG (mode, 6);
       break;
     case 1:
-      result = gen_rtx (REG, mode, 7);
+      result = gen_rtx_REG (mode, 7);
       break;
     case 2:
-      result = gen_rtx (REG, mode, 8);
+      result = gen_rtx_REG (mode, 8);
       break;
     case 3:
-      result = gen_rtx (REG, mode, 9);
+      result = gen_rtx_REG (mode, 9);
       break;
     default:
       result = 0;
@@ -450,7 +411,7 @@ print_operand (file, x, code)
     case 'O':
       if (special_symbolref_operand (x, VOIDmode))
         {
-          char* name;
+          const char *name;
 
 	  if (GET_CODE (x) == SYMBOL_REF)
 	    name = XSTR (x, 0);
@@ -508,8 +469,10 @@ print_operand (file, x, code)
 	  fprintf (file, reg_names[REGNO (x) + 1]);
 	  break;
 	case MEM:
-	  print_operand_address (file,
-				 XEXP (adj_offsettable_operand (x, 4), 0));
+	  x = XEXP (adj_offsettable_operand (x, 4), 0);
+	  print_operand_address (file, x);
+	  if (GET_CODE (x) == CONST_INT)
+	    fprintf (file, "[r0]");
 	  break;
 	  
 	default:
@@ -560,9 +523,8 @@ print_operand (file, x, code)
 	{
 	case MEM:
 	  if (GET_CODE (XEXP (x, 0)) == CONST_INT)
-	    output_address (gen_rtx (PLUS, SImode,
-				     gen_rtx (REG, SImode, 0),
-				     XEXP (x, 0)));
+	    output_address (gen_rtx_PLUS (SImode, gen_rtx (REG, SImode, 0),
+					  XEXP (x, 0)));
 	  else
 	    output_address (XEXP (x, 0));
 	  break;
@@ -634,9 +596,9 @@ print_operand_address (file, addr)
     case SYMBOL_REF:
       if (ENCODED_NAME_P (XSTR (addr, 0)))
         {
-          char* name = XSTR (addr, 0);
-          char* off_name;
-          char* reg_name;
+          const char *name = XSTR (addr, 0);
+          const char *off_name;
+          const char *reg_name;
 
           if (ZDA_NAME_P (name))
             {
@@ -666,9 +628,9 @@ print_operand_address (file, addr)
     case CONST:
       if (special_symbolref_operand (addr, VOIDmode))
         {
-          char* name = XSTR (XEXP (XEXP (addr, 0), 0), 0);
-          char* off_name;
-          char* reg_name;
+          const char *name = XSTR (XEXP (XEXP (addr, 0), 0), 0);
+          const char *off_name;
+          const char *reg_name;
 
           if (ZDA_NAME_P (name))
             {
@@ -789,7 +751,7 @@ output_move_single (operands)
 	return "%S0st%W0 %.,%0";
     }
 
-  fatal_insn ("output_move_single:", gen_rtx (SET, VOIDmode, dst, src));
+  fatal_insn ("output_move_single:", gen_rtx_SET (VOIDmode, dst, src));
   return "";
 }
 
@@ -836,7 +798,7 @@ output_move_double (operands)
 
       for (i = 0; i < 2; i++)
 	{
-	  xop[0] = gen_rtx (REG, SImode, REGNO (dst)+i);
+	  xop[0] = gen_rtx_REG (SImode, REGNO (dst)+i);
 	  xop[1] = GEN_INT (high_low[i]);
 	  output_asm_insn (output_move_single (xop), xop);
 	}
@@ -1090,14 +1052,14 @@ substitute_ep_register (first_insn, last_insn, uses, regno, p_r1, p_ep)
      rtx *p_r1;
      rtx *p_ep;
 {
-  rtx reg = gen_rtx (REG, Pmode, regno);
+  rtx reg = gen_rtx_REG (Pmode, regno);
   rtx insn;
 
   if (!*p_r1)
     {
       regs_ever_live[1] = 1;
-      *p_r1 = gen_rtx (REG, Pmode, 1);
-      *p_ep = gen_rtx (REG, Pmode, 30);
+      *p_r1 = gen_rtx_REG (Pmode, 1);
+      *p_ep = gen_rtx_REG (Pmode, 30);
     }
 
   if (TARGET_DEBUG)
@@ -1153,8 +1115,9 @@ Saved %d bytes (%d uses of register %s) in function %s, starting as insn %d, end
 						   unsignedp))
 			   && ((INTVAL (XEXP (addr, 1))) >= 0))
 		    *p_mem = change_address (*p_mem, VOIDmode,
-					     gen_rtx (PLUS, Pmode,
-						      *p_ep, XEXP (addr, 1)));
+					     gen_rtx_PLUS (Pmode,
+							   *p_ep,
+							   XEXP (addr, 1)));
 		}
 	    }
 	}
@@ -1168,10 +1131,10 @@ Saved %d bytes (%d uses of register %s) in function %s, starting as insn %d, end
       && SET_SRC (PATTERN (insn)) == *p_r1)
     delete_insn (insn);
   else
-    emit_insn_before (gen_rtx (SET, Pmode, *p_r1, *p_ep), first_insn);
+    emit_insn_before (gen_rtx_SET (Pmode, *p_r1, *p_ep), first_insn);
 
-  emit_insn_before (gen_rtx (SET, Pmode, *p_ep, reg), first_insn);
-  emit_insn_before (gen_rtx (SET, Pmode, *p_ep, *p_r1), last_insn);
+  emit_insn_before (gen_rtx_SET (Pmode, *p_ep, reg), first_insn);
+  emit_insn_before (gen_rtx_SET (Pmode, *p_ep, *p_r1), last_insn);
 }
 
 
@@ -1508,8 +1471,6 @@ compute_frame_size (size, p_reg_saved)
      int size;
      long *p_reg_saved;
 {
-  extern int current_function_outgoing_args_size;
-
   return (size
 	  + compute_register_save_size (p_reg_saved)
 	  + current_function_outgoing_args_size);
@@ -1534,10 +1495,10 @@ expand_prologue ()
 
   actual_fsize = compute_frame_size (size, &reg_saved);
 
-  /* Save/setup global registers for interrupt functions right now */
+  /* Save/setup global registers for interrupt functions right now.  */
   if (interrupt_handler)
     {
-      emit_insn (gen_save_interrupt ());
+	emit_insn (gen_save_interrupt ());
       
       actual_fsize -= INTERRUPT_FIXED_SAVE_SIZE;
       
@@ -1557,29 +1518,29 @@ expand_prologue ()
 	  offset = 0;
 	  for (i = 6; i < 10; i++)
 	    {
-	      emit_move_insn (gen_rtx (MEM, SImode,
-				       plus_constant (stack_pointer_rtx,
-						      offset)),
-			      gen_rtx (REG, SImode, i));
+	      emit_move_insn (gen_rtx_MEM (SImode,
+					   plus_constant (stack_pointer_rtx,
+							  offset)),
+			      gen_rtx_REG (SImode, i));
 	      offset += 4;
 	    }
 	}
     }
 
-  /* Identify all of the saved registers */
+  /* Identify all of the saved registers.  */
   num_save = 0;
   default_stack = 0;
   for (i = 1; i < 31; i++)
     {
       if (((1L << i) & reg_saved) != 0)
-	save_regs[num_save++] = gen_rtx (REG, Pmode, i);
+	save_regs[num_save++] = gen_rtx_REG (Pmode, i);
     }
 
   /* If the return pointer is saved, the helper functions also allocate
      16 bytes of stack for arguments to be saved in.  */
   if (((1L << LINK_POINTER_REGNUM) & reg_saved) != 0)
     {
-      save_regs[num_save++] = gen_rtx (REG, Pmode, LINK_POINTER_REGNUM);
+      save_regs[num_save++] = gen_rtx_REG (Pmode, LINK_POINTER_REGNUM);
       default_stack = 16;
     }
 
@@ -1609,28 +1570,30 @@ expand_prologue ()
 	 stack space is allocated.  */
       if (save_func_len < save_normal_len)
 	{
-	  save_all = gen_rtx (PARALLEL, VOIDmode,
-			      rtvec_alloc (num_save + (TARGET_V850 ? 2 : 1)));
-	  XVECEXP (save_all, 0, 0) = gen_rtx (SET, VOIDmode,
-					      stack_pointer_rtx,
-					      gen_rtx (PLUS, Pmode,
-						       stack_pointer_rtx,
-						       GEN_INT (-alloc_stack)));
+	  save_all = gen_rtx_PARALLEL
+	    (VOIDmode,
+	     rtvec_alloc (num_save + (TARGET_V850 ? 2 : 1)));
+
+	  XVECEXP (save_all, 0, 0)
+	    = gen_rtx_SET (VOIDmode,
+			   stack_pointer_rtx,
+			   plus_constant (stack_pointer_rtx, -alloc_stack));
 
 	  if (TARGET_V850)
 	    {
-	      XVECEXP (save_all, 0, num_save + 1)
-		= gen_rtx (CLOBBER, VOIDmode, gen_rtx (REG, Pmode, 10));
+	      XVECEXP (save_all, 0, num_save+1)
+		= gen_rtx_CLOBBER (VOIDmode, gen_rtx_REG (Pmode, 10));
 	    }
 
 	  offset = - default_stack;
 	  for (i = 0; i < num_save; i++)
 	    {
-	      XVECEXP (save_all, 0, i + 1)
-		= gen_rtx (SET, VOIDmode,
-			   gen_rtx (MEM, Pmode,
-				    plus_constant (stack_pointer_rtx, offset)),
-				    save_regs[i]);
+	      XVECEXP (save_all, 0, i+1)
+		= gen_rtx_SET (VOIDmode,
+			       gen_rtx_MEM (Pmode,
+					    plus_constant (stack_pointer_rtx,
+							   offset)),
+			       save_regs[i]);
 	      offset -= 4;
 	    }
 
@@ -1683,18 +1646,18 @@ Saved %d bytes via prologue function (%d vs. %d) for function %s\n",
 	  /* Save the return pointer first.  */
 	  if (num_save > 0 && REGNO (save_regs[num_save-1]) == LINK_POINTER_REGNUM)
 	    {
-	      emit_move_insn (gen_rtx (MEM, SImode,
-				       plus_constant (stack_pointer_rtx,
-						      offset)),
+	      emit_move_insn (gen_rtx_MEM (SImode,
+					   plus_constant (stack_pointer_rtx,
+							  offset)),
 			      save_regs[--num_save]);
 	      offset -= 4;
 	    }
 	  
 	  for (i = 0; i < num_save; i++)
 	    {
-	      emit_move_insn (gen_rtx (MEM, SImode,
-				       plus_constant (stack_pointer_rtx,
-						      offset)),
+	      emit_move_insn (gen_rtx_MEM (SImode,
+					   plus_constant (stack_pointer_rtx,
+							  offset)),
 			      save_regs[i]);
 	      offset -= 4;
 	    }
@@ -1713,7 +1676,7 @@ Saved %d bytes via prologue function (%d vs. %d) for function %s\n",
 			       GEN_INT (-diff)));
       else
 	{
-	  rtx reg = gen_rtx (REG, Pmode, 12);
+	  rtx reg = gen_rtx_REG (Pmode, 12);
 	  emit_move_insn (reg, GEN_INT (-diff));
 	  emit_insn (gen_addsi3 (stack_pointer_rtx, stack_pointer_rtx, reg));
 	}
@@ -1753,27 +1716,29 @@ expand_epilogue ()
   if (frame_pointer_needed)
     emit_move_insn (stack_pointer_rtx, hard_frame_pointer_rtx);
 
-  /* Identify all of the saved registers */
+  /* Identify all of the saved registers.  */
   num_restore = 0;
   default_stack = 0;
   for (i = 1; i < 31; i++)
     {
       if (((1L << i) & reg_saved) != 0)
-	restore_regs[num_restore++] = gen_rtx (REG, Pmode, i);
+	restore_regs[num_restore++] = gen_rtx_REG (Pmode, i);
     }
 
   /* If the return pointer is saved, the helper functions also allocate
      16 bytes of stack for arguments to be saved in.  */
   if (((1L << LINK_POINTER_REGNUM) & reg_saved) != 0)
     {
-      restore_regs[num_restore++] = gen_rtx (REG, Pmode, LINK_POINTER_REGNUM);
+      restore_regs[num_restore++] = gen_rtx_REG (Pmode, LINK_POINTER_REGNUM);
       default_stack = 16;
     }
 
   /* See if we have an insn that restores the particular registers we
      want to.  */
   restore_all = NULL_RTX;
-  if (TARGET_PROLOG_FUNCTION && num_restore > 0
+  
+  if (TARGET_PROLOG_FUNCTION
+      && num_restore > 0
       && actual_fsize >= default_stack
       && !interrupt_handler)
     {
@@ -1785,7 +1750,7 @@ expand_epilogue ()
       if (unalloc_stack)
 	restore_func_len += CONST_OK_FOR_J (unalloc_stack) ? 2 : 4;
 
-      /* see if we would have used ep to restore the registers */
+      /* See if we would have used ep to restore the registers.  */
       if (TARGET_EP && num_restore > 3 && (unsigned)actual_fsize < 255)
 	restore_normal_len = (3 * 2) + (2 * num_restore);
       else
@@ -1796,28 +1761,29 @@ expand_epilogue ()
       /* Don't bother checking if we don't actually save any space.  */
       if (restore_func_len < restore_normal_len)
 	{
-	  restore_all = gen_rtx (PARALLEL, VOIDmode,
-				 rtvec_alloc (num_restore + 2));
-	  XVECEXP (restore_all, 0, 0) = gen_rtx (RETURN, VOIDmode);
+	  restore_all = gen_rtx_PARALLEL (VOIDmode,
+					  rtvec_alloc (num_restore + 2));
+	  XVECEXP (restore_all, 0, 0) = gen_rtx_RETURN (VOIDmode);
 	  XVECEXP (restore_all, 0, 1)
-	    = gen_rtx (SET, VOIDmode, stack_pointer_rtx,
-		       gen_rtx (PLUS, Pmode,
-				stack_pointer_rtx,
-				GEN_INT (alloc_stack)));
+	    = gen_rtx_SET (VOIDmode, stack_pointer_rtx,
+			    gen_rtx_PLUS (Pmode,
+					  stack_pointer_rtx,
+					  GEN_INT (alloc_stack)));
 
 	  offset = alloc_stack - 4;
 	  for (i = 0; i < num_restore; i++)
 	    {
 	      XVECEXP (restore_all, 0, i+2)
-		= gen_rtx (SET, VOIDmode,
-			   restore_regs[i],
-			   gen_rtx (MEM, Pmode,
-				    plus_constant
-				    (stack_pointer_rtx, offset)));
+		= gen_rtx_SET (VOIDmode,
+			       restore_regs[i],
+			       gen_rtx_MEM (Pmode,
+					    plus_constant (stack_pointer_rtx,
+							   offset)));
 	      offset -= 4;
 	    }
 
 	  code = recog (restore_all, NULL_RTX, NULL_PTR);
+	  
 	  if (code >= 0)
 	    {
 	      rtx insn;
@@ -1831,7 +1797,7 @@ expand_epilogue ()
 					   GEN_INT (actual_fsize)));
 		  else
 		    {
-		      rtx reg = gen_rtx (REG, Pmode, 12);
+		      rtx reg = gen_rtx_REG (Pmode, 12);
 		      emit_move_insn (reg, GEN_INT (actual_fsize));
 		      emit_insn (gen_addsi3 (stack_pointer_rtx,
 					     stack_pointer_rtx,
@@ -1855,7 +1821,7 @@ Saved %d bytes via epilogue function (%d vs. %d) in function %s\n",
     }
 
   /* If no epilog save function is available, restore the registers the
-     old fashioned way (one by one). */
+     old fashioned way (one by one).  */
   if (!restore_all)
     {
       /* If the stack is large, we need to cut it down in 2 pieces.  */
@@ -1870,14 +1836,14 @@ Saved %d bytes via epilogue function (%d vs. %d) in function %s\n",
 	  int diff;
 
 	  diff = actual_fsize - ((interrupt_handler) ? 0 : init_stack_free);
-	  
+
 	  if (CONST_OK_FOR_K (diff))
 	    emit_insn (gen_addsi3 (stack_pointer_rtx,
 				   stack_pointer_rtx,
 				   GEN_INT (diff)));
 	  else
 	    {
-	      rtx reg = gen_rtx (REG, Pmode, 12);
+	      rtx reg = gen_rtx_REG (Pmode, 12);
 	      emit_move_insn (reg, GEN_INT (diff));
 	      emit_insn (gen_addsi3 (stack_pointer_rtx,
 				     stack_pointer_rtx,
@@ -1893,7 +1859,7 @@ Saved %d bytes via epilogue function (%d vs. %d) in function %s\n",
 	}
       else
 	{
-	  /* Restore registers from the beginning of the stack frame */
+	  /* Restore registers from the beginning of the stack frame.  */
 	  offset = init_stack_free - 4;
 
 	  /* Restore the return pointer first.  */
@@ -1901,19 +1867,20 @@ Saved %d bytes via epilogue function (%d vs. %d) in function %s\n",
 	      && REGNO (restore_regs [num_restore - 1]) == LINK_POINTER_REGNUM)
 	    {
 	      emit_move_insn (restore_regs[--num_restore],
-			      gen_rtx (MEM, SImode,
-				       plus_constant (stack_pointer_rtx,
-						      offset)));
+			      gen_rtx_MEM (SImode,
+					   plus_constant (stack_pointer_rtx,
+							  offset)));
 	      offset -= 4;
 	    }
 
 	  for (i = 0; i < num_restore; i++)
 	    {
 	      emit_move_insn (restore_regs[i],
-			      gen_rtx (MEM, SImode,
-				       plus_constant (stack_pointer_rtx,
-						      offset)));
+			      gen_rtx_MEM (SImode,
+					   plus_constant (stack_pointer_rtx,
+							  offset)));
 
+	      emit_insn (gen_rtx_USE (VOIDmode, restore_regs[i]));
 	      offset -= 4;
 	    }
 
@@ -1955,24 +1922,24 @@ notice_update_cc (body, insn)
     case CC_NONE_0HIT:
       /* Insn does not change CC, but the 0'th operand has been changed.  */
       if (cc_status.value1 != 0
-	  && reg_overlap_mentioned_p (recog_operand[0], cc_status.value1))
+	  && reg_overlap_mentioned_p (recog_data.operand[0], cc_status.value1))
 	cc_status.value1 = 0;
       break;
 
     case CC_SET_ZN:
-      /* Insn sets the Z,N flags of CC to recog_operand[0].
+      /* Insn sets the Z,N flags of CC to recog_data.operand[0].
 	 V,C is in an unusable state.  */
       CC_STATUS_INIT;
       cc_status.flags |= CC_OVERFLOW_UNUSABLE | CC_NO_CARRY;
-      cc_status.value1 = recog_operand[0];
+      cc_status.value1 = recog_data.operand[0];
       break;
 
     case CC_SET_ZNV:
-      /* Insn sets the Z,N,V flags of CC to recog_operand[0].
+      /* Insn sets the Z,N,V flags of CC to recog_data.operand[0].
 	 C is in an unusable state.  */
       CC_STATUS_INIT;
       cc_status.flags |= CC_NO_CARRY;
-      cc_status.value1 = recog_operand[0];
+      cc_status.value1 = recog_data.operand[0];
       break;
 
     case CC_COMPARE:
@@ -2123,7 +2090,7 @@ void
 v850_encode_data_area (decl)
      tree decl;
 {
-  char * str = XSTR (XEXP (DECL_RTL (decl), 0), 0);
+  const char *str = XSTR (XEXP (DECL_RTL (decl), 0), 0);
   int    len = strlen (str);
   char * newstr;
 
@@ -2132,7 +2099,7 @@ v850_encode_data_area (decl)
     {
       if (DECL_SECTION_NAME (decl))
 	{
-	  char * name = TREE_STRING_POINTER (DECL_SECTION_NAME (decl));
+	  const char *name = TREE_STRING_POINTER (DECL_SECTION_NAME (decl));
 	  
 	  if (streq (name, ".zdata") || streq (name, ".zbss"))
 	    v850_set_data_area (decl, DATA_AREA_ZDA);
@@ -2959,8 +2926,8 @@ parse_ghs_pragma_token (string)
 /* Handle the parsing of an entire GHS pragma.  */
 int
 v850_handle_pragma (p_getc, p_ungetc, name)
-     int (*  p_getc) PROTO ((void));
-     void (* p_ungetc) PROTO ((int));
+     int (*  p_getc) PARAMS ((void));
+     void (* p_ungetc) PARAMS ((int));
      char *  name;
 {
   /* Parse characters in the input stream until:
@@ -3018,8 +2985,7 @@ v850_handle_pragma (p_getc, p_ungetc, name)
 	  /* Read string.  */
 	  do
 	    * buff ++ = (c = p_getc ());
-	  while (c != EOF && isascii (c)
-		 && (isalnum (c) || c == '_' || c == '.' || c == ' ')
+	  while (c != EOF && (ISALNUM (c) || c == '_' || c == '.' || c == ' ')
 		 && (buff < buffer + 126));
 	  
 	  if (c != '"')
@@ -3029,8 +2995,7 @@ v850_handle_pragma (p_getc, p_ungetc, name)
 	  break;
 
 	default:
-	  while (c != EOF && isascii (c)
-		 && (isalnum (c) || c == '_' || c == '.')
+	  while (c != EOF && (ISALNUM (c) || c == '_' || c == '.')
 		 && (buff < buffer + 126))
 	    * buff ++ = (c = p_getc ());
 	  break;
@@ -3147,3 +3112,45 @@ v850_set_default_decl_attr (decl)
 	}
     }
 }
+
+/* Implement `va_arg'.  */
+
+rtx
+v850_va_arg (valist, type)
+     tree valist, type;
+{
+  HOST_WIDE_INT size, rsize;
+  tree addr, incr;
+  rtx addr_rtx;
+  int indirect;
+
+  /* Round up sizeof(type) to a word.  */
+  size = int_size_in_bytes (type);
+  rsize = (size + UNITS_PER_WORD - 1) & -UNITS_PER_WORD;
+  indirect = 0;
+
+  if (size > 8)
+    {
+      size = rsize = UNITS_PER_WORD;
+      indirect = 1;
+    }
+
+  addr = save_expr (valist);
+  incr = fold (build (PLUS_EXPR, ptr_type_node, valist,
+		      build_int_2 (rsize, 0)));
+
+  incr = build (MODIFY_EXPR, ptr_type_node, valist, incr);
+  TREE_SIDE_EFFECTS (incr) = 1;
+  expand_expr (incr, const0_rtx, VOIDmode, EXPAND_NORMAL);
+
+  addr_rtx = expand_expr (addr, NULL, Pmode, EXPAND_NORMAL);
+
+  if (indirect)
+    {
+      addr_rtx = force_reg (Pmode, addr_rtx);
+      addr_rtx = gen_rtx_MEM (Pmode, addr_rtx);
+      MEM_ALIAS_SET (addr_rtx) = get_varargs_alias_set ();
+    }
+
+  return addr_rtx;
+}
