@@ -1,6 +1,7 @@
 /* Specialized bits of code needed to support construction and
    destruction of file-scope objects in C++ code.
-   Copyright (C) 1991, 94, 95, 96, 97, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1994, 1995, 1996, 1997, 1998,
+   1999, 2000 Free Software Foundation, Inc.
    Contributed by Ron Guilmette (rfg@monkeys.com).
 
 This file is part of GNU CC.
@@ -10,6 +11,15 @@ it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
+In addition to the permissions in the GNU General Public License, the
+Free Software Foundation gives you unlimited permission to link the
+compiled version of this file into combinations with other programs,
+and to distribute those combinations without any restriction coming
+from the use of this file.  (The General Public License restrictions
+do apply in other respects; for example, they cover modification of
+the file, and distribution when not linked into a combine
+executable.)
+
 GNU CC is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -20,12 +30,6 @@ along with GNU CC; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
-/* As a special exception, if you link this library with files
-   compiled with GCC to produce an executable, this does not cause
-   the resulting executable to be covered by the GNU General Public License.
-   This exception does not however invalidate any other reasons why
-   the executable file might be covered by the GNU General Public License.  */
-
 /* This file is a bit like libgcc1.c/libgcc2.c in that it is compiled
    multiple times and yields multiple .o files.
 
@@ -51,9 +55,11 @@ Boston, MA 02111-1307, USA.  */
    compiled for the target, and hence definitions concerning only the host
    do not apply.  */
 
+#include "auto-host.h"
 #include "tm.h"
+#include "tsystem.h"
+
 #include "defaults.h"
-#include <stddef.h>
 #include "frame.h"
 
 /* We do not want to add the weak attribute to the declarations of these
@@ -108,9 +114,6 @@ extern void *__deregister_frame_info (void *)
 #ifndef DTORS_SECTION_ASM_OP
 #define DTORS_SECTION_ASM_OP	".section\t.dtors,\"aw\""
 #endif
-#if !defined (EH_FRAME_SECTION_ASM_OP) && defined (DWARF2_UNWIND_INFO) && defined(ASM_OUTPUT_SECTION_NAME)
-#define EH_FRAME_SECTION_ASM_OP	".section\t.eh_frame,\"aw\""
-#endif
 
 #ifdef OBJECT_FORMAT_ELF
 
@@ -122,9 +125,6 @@ typedef void (*func_ptr) (void);
 
 #include "gbl-ctors.h"
 
-#ifndef ON_EXIT
-#define ON_EXIT(a, b)
-#endif
 #define STATIC
 
 #endif /* OBJECT_FORMAT_ELF */
@@ -135,6 +135,29 @@ typedef void (*func_ptr) (void);
 
 #ifdef OBJECT_FORMAT_ELF
 
+/* Declare the __dso_handle variable.  It should have a unique value
+   in every shared-object; in a main program its value is zero.  The
+   object should in any case be protected.  This means the instance
+   in one DSO or the main program is not used in another object.  The
+   dynamic linker takes care of this.  */
+
+/* XXX Ideally the following should be implemented using
+       __attribute__ ((__visibility__ ("hidden")))
+   but the __attribute__ support is not yet there.  */
+#ifdef HAVE_GAS_HIDDEN
+asm (".hidden\t__dso_handle");
+#endif
+
+#ifdef CRTSTUFFS_O
+void *__dso_handle = &__dso_handle;
+#else
+void *__dso_handle = 0;
+#endif
+
+/* The __cxa_finalize function may not be available so we use only a
+   weak declaration.  */
+extern void __cxa_finalize (void *) TARGET_ATTRIBUTE_WEAK;
+
 /* Run all the global destructors on exit from the program.  */
  
 /* Some systems place the number of pointers in the first word of the
@@ -165,6 +188,11 @@ __do_global_dtors_aux (void)
   if (completed)
     return;
 
+#ifdef CRTSTUFFS_O
+  if (__cxa_finalize)
+    __cxa_finalize (__dso_handle);
+#endif
+
   while (*p)
     {
       p++;
@@ -226,7 +254,7 @@ init_dummy (void)
    INVOKE__main is defined.  This has the additional effect of forcing cc1
    to switch to the .text section.  */
 
-static void __do_global_ctors_aux ();
+static void __do_global_ctors_aux (void);
 void
 __do_global_ctors (void)
 {
@@ -265,7 +293,7 @@ __do_global_ctors_aux (void)	/* prologue goes in .init section */
 #endif
   asm (TEXT_SECTION_ASM_OP);	/* don't put epilogue and body in .init */
   DO_GLOBAL_CTORS_BODY;
-  ON_EXIT (__do_global_dtors, 0);
+  atexit (__do_global_dtors);
 }
 
 #endif /* OBJECT_FORMAT_ELF */
@@ -391,7 +419,7 @@ init_dummy (void)
     extern char **__environ;
 
     ___brk_addr = __environ;
-    atexit ();
+    atexit (0);
   }
 #endif
 }
@@ -424,7 +452,7 @@ __do_global_ctors_aux (void)	/* prologue goes in .text section */
 {
   asm (INIT_SECTION_ASM_OP);
   DO_GLOBAL_CTORS_BODY;
-  ON_EXIT (__do_global_dtors, 0);
+  atexit (__do_global_dtors);
 }				/* epilogue and body go in .init section */
 
 #ifdef FORCE_INIT_SECTION_ALIGN
@@ -519,7 +547,7 @@ extern const struct section *
 
 #ifdef CRT_BEGIN
 
-static void __reg_frame_ctor () __attribute__ ((constructor));
+static void __reg_frame_ctor (void) __attribute__ ((constructor));
 
 static void
 __reg_frame_ctor (void)
@@ -536,10 +564,9 @@ __reg_frame_ctor (void)
 
 #ifdef CRT_END
 
-static void __dereg_frame_dtor () __attribute__ ((destructor));
+static void __dereg_frame_dtor (void) __attribute__ ((destructor));
 
-static
-void
+static void
 __dereg_frame_dtor (void)
 {
   const struct section *eh_frame;
