@@ -234,7 +234,7 @@ int
 main (int argc, char **argv)
 {
   char buf[MAXNAMELEN];
-  char last_buf[MAXNAMELEN] = "";
+  char last_buf[MAXNAMELEN];
   char kwname[MAXNAMELEN];
   char routine[32];
   char type[32];
@@ -256,6 +256,8 @@ main (int argc, char **argv)
   int cc;
   bool do_exit = FALSE;
 
+  last_buf[0] = '\0';
+  
   for (i = 0; ((size_t) i) < ARRAY_SIZE (names); ++i)
     {				/* Initialize length/name ordered list roots. */
       names[i].first = (name) &names[i];
@@ -267,7 +269,7 @@ main (int argc, char **argv)
   if (argc != 4)
     {
       fprintf (stderr, "Command form: fini input output-code output-include\n");
-      exit (1);
+      return (1);
     }
 
   input_name = argv[1];
@@ -278,21 +280,21 @@ main (int argc, char **argv)
   if (in == NULL)
     {
       fprintf (stderr, "Cannot open \"%s\"\n", input_name);
-      exit (1);
+      return (1);
     }
   out = fopen (output_name, "w");
   if (out == NULL)
     {
       fclose (in);
       fprintf (stderr, "Cannot open \"%s\"\n", output_name);
-      exit (1);
+      return (1);
     }
   incl = fopen (include_name, "w");
   if (incl == NULL)
     {
       fclose (in);
       fprintf (stderr, "Cannot open \"%s\"\n", include_name);
-      exit (1);
+      return (1);
     }
 
   /* Get past the initial block-style comment (man, this parsing code is just
@@ -316,7 +318,7 @@ main (int argc, char **argv)
       else
 	{
 	  assert ("EOF too soon!" == NULL);
-	  exit (1);
+	  return (1);
 	}
     }
 
@@ -381,8 +383,8 @@ main (int argc, char **argv)
 	  cc = buf[i];
 	  if (ISALPHA (cc))
 	    {
-	      newname->name_uc[i] = toupper (cc);
-	      newname->name_lc[i] = tolower (cc);
+	      newname->name_uc[i] = TOUPPER (cc);
+	      newname->name_lc[i] = TOLOWER (cc);
 	      newname->name_ic[i] = cc;
 	    }
 	  else
@@ -441,7 +443,7 @@ main (int argc, char **argv)
 #endif
 
   if (do_exit)
-    exit (1);
+    return (1);
 
   /* First output the #include file. */
 
@@ -624,7 +626,7 @@ typedef enum %s_ %s;\n\
     fclose (incl);
   if (in != stdin)
     fclose (in);
-  exit (0);
+  return (0);
 }
 
 void
