@@ -1,5 +1,5 @@
 /* Functions related to building classes and their related objects.
-   Copyright (C) 1996, 97-98, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
 
 This file is part of GNU CC.
 
@@ -36,21 +36,171 @@ The Free Software Foundation is independent of Sun Microsystems, Inc.  */
 #include "output.h"
 #include "parse.h"
 
-static tree mangle_class_field PROTO ((tree class));
-static tree make_method_value PROTO ((tree));
-static tree build_java_method_type PROTO ((tree, tree, int));
-static int32 hashUtf8String PROTO ((const char *, int));
-static tree make_field_value PROTO ((tree));
-static tree get_dispatch_vector PROTO ((tree));
-static tree get_dispatch_table PROTO ((tree, tree));
-static void append_gpp_mangled_type PROTO ((struct obstack *, tree));
-static tree mangle_static_field PROTO ((tree));
+static tree mangle_class_field PARAMS ((tree class));
+static tree make_method_value PARAMS ((tree));
+static tree build_java_method_type PARAMS ((tree, tree, int));
+static int32 hashUtf8String PARAMS ((const char *, int));
+static tree make_field_value PARAMS ((tree));
+static tree get_dispatch_vector PARAMS ((tree));
+static tree get_dispatch_table PARAMS ((tree, tree));
+static void append_gpp_mangled_type PARAMS ((struct obstack *, tree));
+static tree mangle_static_field PARAMS ((tree));
+static void add_interface_do PARAMS ((tree, tree, int));
+static tree maybe_layout_super_class PARAMS ((tree, tree));
+static int assume_compiled PARAMS ((const char *));
+static struct hash_entry *init_test_hash_newfunc PARAMS ((struct hash_entry *,
+							  struct hash_table *,
+							  hash_table_key));
 
 static rtx registerClass_libfunc;
 
 extern struct obstack permanent_obstack;
 extern struct obstack temporary_obstack;
 
+/* The compiler generates different code depending on whether or not
+   it can assume certain classes have been compiled down to native
+   code or not.  The compiler options -fassume-compiled= and
+   -fno-assume-compiled= are used to create a tree of
+   assume_compiled_node objects.  This tree is queried to determine if
+   a class is assume to be compiled or not.  Each node in the tree
+   represents either a package or a specific class.  */
+
+typedef struct assume_compiled_node_struct
+{
+  /* The class or package name.  */
+  const char *ident;
+
+  /* Non-zero if this represents an exclusion.  */
+  int excludep;
+
+  /* Pointers to other nodes in the tree.  */
+  struct assume_compiled_node_struct *parent;
+  struct assume_compiled_node_struct *sibling;
+  struct assume_compiled_node_struct *child;
+} assume_compiled_node;
+
+static assume_compiled_node *find_assume_compiled_node
+			PARAMS ((assume_compiled_node *, const char *));
+
+/* This is the root of the include/exclude tree.  */
+
+static assume_compiled_node *assume_compiled_tree;
+
+/* Return the node that most closely represents the class whose name
+   is IDENT.  Start the search from NODE.  Return NULL if an
+   appropriate node does not exist.  */
+
+static assume_compiled_node *
+find_assume_compiled_node (node, ident)
+     assume_compiled_node *node;
+     const char *ident;
+{
+  while (node)
+    {
+      size_t node_ident_length = strlen (node->ident);
+
+      /* node_ident_length is zero at the root of the tree.  If the
+	 identifiers are the same length, then we have matching
+	 classes.  Otherwise check if we've matched an enclosing
+	 package name.  */
+
+      if (node_ident_length == 0
+	  || (strncmp (ident, node->ident, node_ident_length) == 0
+	      && (strlen (ident) == node_ident_length
+		  || ident[node_ident_length] == '.')))
+	{
+	  /* We've found a match, however, there might be a more
+             specific match.  */
+
+	  assume_compiled_node *found = find_assume_compiled_node (node->child,
+								   ident);
+	  if (found)
+	    return found;
+	  else
+	    return node;
+	}
+
+      /* No match yet.  Continue through the sibling list.  */
+      node = node->sibling;
+    }
+
+  /* No match at all in this tree.  */
+  return NULL;
+}
+
+/* Add a new IDENT to the include/exclude tree.  It's an exclusion
+   if EXCLUDEP is non-zero.  */
+
+void
+add_assume_compiled (ident, excludep)
+     const char *ident;
+     int excludep;
+{
+  assume_compiled_node *parent;
+  assume_compiled_node *node = 
+    (assume_compiled_node *) xmalloc (sizeof (assume_compiled_node));
+
+  node->ident = xstrdup (ident);
+  node->excludep = excludep;
+  node->child = NULL;
+
+  /* Create the root of the tree if it doesn't exist yet.  */
+
+  if (NULL == assume_compiled_tree)
+    {
+      assume_compiled_tree = 
+	(assume_compiled_node *) xmalloc (sizeof (assume_compiled_node));
+      assume_compiled_tree->ident = "";
+      assume_compiled_tree->excludep = 0;
+      assume_compiled_tree->sibling = NULL;
+      assume_compiled_tree->child = NULL;
+      assume_compiled_tree->parent = NULL;
+    }
+
+  /* Calling the function with the empty string means we're setting
+     excludep for the root of the hierarchy.  */
+
+  if (0 == ident[0])
+    {
+      assume_compiled_tree->excludep = excludep;
+      return;
+    }
+
+  /* Find the parent node for this new node.  PARENT will either be a
+     class or a package name.  Adjust PARENT accordingly.  */
+
+  parent = find_assume_compiled_node (assume_compiled_tree, ident);
+  if (ident[strlen (parent->ident)] != '.')
+    parent = parent->parent;
+
+  /* Insert NODE into the tree.  */
+
+  node->parent = parent;
+  node->sibling = parent->child;
+  parent->child = node;
+}
+
+/* Returns non-zero if IDENT is the name of a class that the compiler
+   should assume has been compiled to FIXME  */
+
+static int
+assume_compiled (ident)
+     const char *ident;
+{
+  assume_compiled_node *i;
+  int result;
+  
+  if (NULL == assume_compiled_tree)
+    return 1;
+
+  i = find_assume_compiled_node (assume_compiled_tree,
+				 ident);
+
+  result = ! i->excludep;
+  
+  return (result);
+}
+
 /* Return an IDENTIFIER_NODE the same as (OLD_NAME, OLD_LENGTH).
    except that characters matching OLD_CHAR are substituted by NEW_CHAR.
    Also, PREFIX is prepended, and SUFFIX is appended. */
@@ -135,6 +285,7 @@ make_class ()
 #else
   TYPE_BINFO (type) = make_tree_vec (6);
 #endif
+  MAYBE_CREATE_TYPE_TYPE_LANG_SPECIFIC (type);
   pop_obstacks ();
 
   return type;
@@ -169,7 +320,7 @@ push_class (class_type, class_name)
      tree class_type, class_name;
 {
   tree decl, signature;
-  char *save_input_filename = input_filename;
+  const char *save_input_filename = input_filename;
   int save_lineno = lineno;
   tree source_name = identifier_subst (class_name, "", '.', '/', ".java");
   push_obstacks (&permanent_obstack, &permanent_obstack);
@@ -242,12 +393,15 @@ set_super_info (access_flags, this_class, super_class, interfaces_count)
       CLASS_HAS_SUPER (this_class) = 1;
     }
   pop_obstacks ();
-  
+
   if (access_flags & ACC_PUBLIC)    CLASS_PUBLIC (class_decl) = 1;
   if (access_flags & ACC_FINAL)     CLASS_FINAL (class_decl) = 1;
   if (access_flags & ACC_SUPER)     CLASS_SUPER (class_decl) = 1;
   if (access_flags & ACC_INTERFACE) CLASS_INTERFACE (class_decl) = 1;
   if (access_flags & ACC_ABSTRACT)  CLASS_ABSTRACT (class_decl) = 1;
+  if (access_flags & ACC_STATIC)    CLASS_STATIC (class_decl) = 1;
+  if (access_flags & ACC_PRIVATE)   CLASS_PRIVATE (class_decl) = 1;
+  if (access_flags & ACC_PROTECTED) CLASS_PROTECTED (class_decl) = 1;
 }
 
 /* Return length of inheritance chain of CLAS, where java.lang.Object is 0,
@@ -260,6 +414,8 @@ class_depth (clas)
   int depth = 0;
   if (! CLASS_LOADED_P (clas))
     load_class (clas, 1);
+  if (TYPE_SIZE (clas) == error_mark_node)
+    return -1;
   while (clas != object_type_node)
     {
       depth++;
@@ -311,6 +467,51 @@ inherits_from_p (type1, type2)
   return 0;
 }
 
+/* Return a 1 iff TYPE1 is an enclosing context for TYPE2 */
+
+int
+enclosing_context_p (type1, type2)
+     tree type1, type2;
+{
+  if (!INNER_CLASS_TYPE_P (type2))
+    return 0;
+
+  for (type2 = TREE_TYPE (DECL_CONTEXT (TYPE_NAME (type2)));
+       type2; 
+       type2 = (INNER_CLASS_TYPE_P (type2) ?
+		TREE_TYPE (DECL_CONTEXT (TYPE_NAME (type2))) : NULL_TREE))
+    {
+      if (type2 == type1)
+	return 1;
+    }
+
+  return 0;
+}
+
+/* Return 1 iff there exists a common enclosing context between TYPE1
+   and TYPE2.  */
+
+int common_enclosing_context_p (type1, type2)
+     tree type1, type2;
+{
+  if (!PURE_INNER_CLASS_TYPE_P (type1) || !PURE_INNER_CLASS_TYPE_P (type2))
+    return 0;
+  
+  for (type1 = TREE_TYPE (DECL_CONTEXT (TYPE_NAME (type1))); type1; 
+       type1 = (PURE_INNER_CLASS_TYPE_P (type1) ?
+		TREE_TYPE (DECL_CONTEXT (TYPE_NAME (type1))) : NULL_TREE))
+    {
+      tree current;
+      for (current = TREE_TYPE (DECL_CONTEXT (TYPE_NAME (type2))); current;
+	   current = (PURE_INNER_CLASS_TYPE_P (current) ?
+		      TREE_TYPE (DECL_CONTEXT (TYPE_NAME (current))) : 
+		      NULL_TREE))
+	if (type1 == current)
+	  return 1;
+    }
+  return 0;
+}
+
 static void
 add_interface_do (basetype_vec, interface_class, i)
      tree basetype_vec, interface_class;
@@ -399,6 +600,43 @@ build_java_method_type (fntype, this_class, access_flags)
   return build_method_type (CLASS_TO_HANDLE_TYPE (this_class), fntype);
 }
 
+static struct hash_entry *
+init_test_hash_newfunc (entry, table, string)
+     struct hash_entry *entry;
+     struct hash_table *table;
+     hash_table_key string ATTRIBUTE_UNUSED;
+{
+  struct init_test_hash_entry *ret = (struct init_test_hash_entry *) entry;
+  if (ret == NULL)
+    {
+      ret = ((struct init_test_hash_entry *)
+	     hash_allocate (table, sizeof (struct init_test_hash_entry)));
+      if (ret == NULL)
+	return NULL;
+    }
+  ret->init_test_decl = 0;
+  return (struct hash_entry *) ret;
+}
+
+/* Hash table helpers. Also reused in find_applicable_accessible_methods_list
+   (parse.y). The hash of a tree node is it's pointer value,
+   comparison is direct. */
+
+unsigned long
+java_hash_hash_tree_node (k)
+     hash_table_key k;
+{
+  return (long) k;
+}
+
+boolean
+java_hash_compare_tree_node (k1, k2)
+     hash_table_key k1;
+     hash_table_key k2;
+{
+  return ((char*) k1 == (char*) k2);
+}
+
 tree
 add_method_1 (handle_class, access_flags, name, function_type)
      tree handle_class;
@@ -419,16 +657,28 @@ add_method_1 (handle_class, access_flags, name, function_type)
     = (struct lang_decl *) permalloc (sizeof (struct lang_decl));
   bzero ((PTR) DECL_LANG_SPECIFIC (fndecl), sizeof (struct lang_decl));
 
+  /* Initialize the static initializer test table.  */
+  hash_table_init (&DECL_FUNCTION_INIT_TEST_TABLE (fndecl),
+		   init_test_hash_newfunc, java_hash_hash_tree_node, 
+		   java_hash_compare_tree_node);
+
   TREE_CHAIN (fndecl) = TYPE_METHODS (handle_class);
   TYPE_METHODS (handle_class) = fndecl;
   pop_obstacks ();
 
   if (access_flags & ACC_PUBLIC) METHOD_PUBLIC (fndecl) = 1;
   if (access_flags & ACC_PROTECTED) METHOD_PROTECTED (fndecl) = 1;
-  if (access_flags & ACC_PRIVATE) METHOD_PRIVATE (fndecl) = 1;
-  if (access_flags & ACC_NATIVE) METHOD_NATIVE (fndecl) = 1;
-  if (access_flags & ACC_STATIC) METHOD_STATIC (fndecl) = 1;
-  if (access_flags & ACC_FINAL) METHOD_FINAL (fndecl) = 1;
+  if (access_flags & ACC_PRIVATE)
+    METHOD_PRIVATE (fndecl) = DECL_INLINE (fndecl) = 1;
+  if (access_flags & ACC_NATIVE)
+    {
+      METHOD_NATIVE (fndecl) = 1;
+      DECL_EXTERNAL (fndecl) = 1;
+    }
+  if (access_flags & ACC_STATIC) 
+    METHOD_STATIC (fndecl) = DECL_INLINE (fndecl) = 1;
+  if (access_flags & ACC_FINAL) 
+    METHOD_FINAL (fndecl) = DECL_INLINE (fndecl) = 1;
   if (access_flags & ACC_SYNCHRONIZED) METHOD_SYNCHRONIZED (fndecl) = 1;
   if (access_flags & ACC_ABSTRACT) METHOD_ABSTRACT (fndecl) = 1;
   if (access_flags & ACC_TRANSIENT) METHOD_TRANSIENT (fndecl) = 1;
@@ -448,7 +698,7 @@ add_method (this_class, access_flags, name, method_sig)
 {
   tree handle_class = CLASS_TO_HANDLE_TYPE (this_class);
   tree function_type, fndecl;
-  unsigned char *sig = (unsigned char*)IDENTIFIER_POINTER (method_sig);
+  const unsigned char *sig = (const unsigned char*)IDENTIFIER_POINTER (method_sig);
   push_obstacks (&permanent_obstack, &permanent_obstack);
   if (sig[0] != '(')
     fatal ("bad method signature");
@@ -555,7 +805,7 @@ tree
 build_utf8_ref (name)
      tree name;
 {
-  char* name_ptr = IDENTIFIER_POINTER(name);
+  const char * name_ptr = IDENTIFIER_POINTER(name);
   int name_len = IDENTIFIER_LENGTH(name);
   char buf[60];
   char *buf_ptr;
@@ -648,6 +898,7 @@ build_class_ref (type)
 	      push_obstacks (&permanent_obstack, &permanent_obstack);
 	      decl = build_decl (VAR_DECL, decl_name, class_type_node);
 	      DECL_SIZE (decl) = TYPE_SIZE (class_type_node);
+	      DECL_SIZE_UNIT (decl) = TYPE_SIZE_UNIT (class_type_node);
 	      TREE_STATIC (decl) = 1;
 	      TREE_PUBLIC (decl) = 1;
 	      DECL_IGNORED_P (decl) = 1;
@@ -662,7 +913,7 @@ build_class_ref (type)
 	}
       else
 	{
-	  char *name;
+	  const char *name;
 	  char buffer[25];
 	  if (flag_emit_class_files)
 	    {
@@ -752,8 +1003,7 @@ build_static_field_ref (fdecl)
   else
     {
       /* Compile as:
-       * *(FTYPE*)build_class_ref(FCLASS)->fields[INDEX].info.addr
-       */
+       * *(FTYPE*)build_class_ref(FCLASS)->fields[INDEX].info.addr */
       static tree fields_ident = NULL_TREE;
       static tree info_ident = NULL_TREE;
       tree ref = build_class_ref (fclass);
@@ -824,6 +1074,12 @@ get_access_flags_from_decl (decl)
 	access_flags |= ACC_INTERFACE;
       if (CLASS_ABSTRACT (decl))
 	access_flags |= ACC_ABSTRACT;
+      if (CLASS_STATIC (decl))
+	access_flags |= ACC_STATIC;
+      if (CLASS_PRIVATE (decl))
+	access_flags |= ACC_PRIVATE;
+      if (CLASS_PROTECTED (decl))
+	access_flags |= ACC_PROTECTED;
       return access_flags;
     }
   if (TREE_CODE (decl) == FUNCTION_DECL)
@@ -855,10 +1111,11 @@ static tree
 make_field_value (fdecl)
   tree fdecl;
 {
-  tree finit, info;
-  int bsize, flags;
+  tree finit;
+  int flags;
   tree type = TREE_TYPE (fdecl);
   int resolved = is_compiled_class (type);
+
   START_RECORD_CONSTRUCTOR (finit, field_type_node);
   PUSH_FIELD_VALUE (finit, "name", build_utf8_ref (DECL_NAME (fdecl)));
   if (resolved)
@@ -866,33 +1123,30 @@ make_field_value (fdecl)
   else
     {
       tree signature = build_java_signature (type);
+
       type = build_utf8_ref (unmangle_classname 
-			     (IDENTIFIER_POINTER(signature),
-			      IDENTIFIER_LENGTH(signature)));
+			     (IDENTIFIER_POINTER (signature),
+			      IDENTIFIER_LENGTH (signature)));
     }
   PUSH_FIELD_VALUE (finit, "type", type);
+
   flags = get_access_flags_from_decl (fdecl);
   if (! resolved)
     flags |= 0x8000 /* FIELD_UNRESOLVED_FLAG */;
+
   PUSH_FIELD_VALUE (finit, "accflags", build_int_2 (flags, 0));
-  bsize = TREE_INT_CST_LOW (TYPE_SIZE (TREE_TYPE (fdecl))) / BITS_PER_UNIT;
-  PUSH_FIELD_VALUE (finit, "bsize", build_int_2 (bsize, 0));
-  if (FIELD_STATIC (fdecl))
-    {
-      tree cfield = TREE_CHAIN (TYPE_FIELDS(field_info_union_node));
-      tree faddr = build_address_of (build_static_field_ref (fdecl));
-      info = build (CONSTRUCTOR, field_info_union_node, NULL_TREE,
-		    build_tree_list (cfield, faddr));
-    }
-  else
-    {
-      int boffset
-	= TREE_INT_CST_LOW (DECL_FIELD_BITPOS (fdecl)) / BITS_PER_UNIT;
-      info = build (CONSTRUCTOR, field_info_union_node, NULL_TREE,
-		    build_tree_list (TYPE_FIELDS(field_info_union_node),
-				     build_int_2 (boffset, 0)));
-    }
-  PUSH_FIELD_VALUE (finit, "info", info);
+  PUSH_FIELD_VALUE (finit, "bsize", TYPE_SIZE_UNIT (TREE_TYPE (fdecl)));
+
+  PUSH_FIELD_VALUE
+    (finit, "info",
+     build (CONSTRUCTOR, field_info_union_node, NULL_TREE,
+	    build_tree_list
+	    ((FIELD_STATIC (fdecl)
+	      ? TREE_CHAIN (TYPE_FIELDS (field_info_union_node))
+	      : TYPE_FIELDS (field_info_union_node)),
+	     (FIELD_STATIC (fdecl)
+	      ? build_address_of (build_static_field_ref (fdecl))
+	      : byte_position (fdecl)))));
 
   FINISH_RECORD_CONSTRUCTOR (finit);
   return finit;
@@ -935,29 +1189,28 @@ get_dispatch_vector (type)
   tree vtable = TYPE_VTABLE (type);
   if (vtable == NULL)
     {
-      int i;
+      HOST_WIDE_INT i;
       tree method;
       tree super = CLASSTYPE_SUPER (type);
-      int nvirtuals = TREE_INT_CST_LOW (TYPE_NVIRTUALS (type));
+      HOST_WIDE_INT nvirtuals = tree_low_cst (TYPE_NVIRTUALS (type), 0);
       vtable = make_tree_vec (nvirtuals);
       TYPE_VTABLE (type) = vtable;
       if (super != NULL_TREE)
 	{
 	  tree super_vtable = get_dispatch_vector (super);
-	  for ( i = TREE_INT_CST_LOW (TYPE_NVIRTUALS (super));  --i >= 0; )
+
+	  for (i = tree_low_cst (TYPE_NVIRTUALS (super), 0); --i >= 0; )
 	    TREE_VEC_ELT (vtable, i) = TREE_VEC_ELT (super_vtable, i);
 	}
+
       for (method = TYPE_METHODS (type);  method != NULL_TREE;
 	   method = TREE_CHAIN (method))
-	{
-	  if (DECL_VINDEX (method) != NULL_TREE
-	      && TREE_CODE (DECL_VINDEX (method)) == INTEGER_CST)
-	    {
-	      TREE_VEC_ELT (vtable, TREE_INT_CST_LOW (DECL_VINDEX (method)))
-		= method;
-	    }
-	}
+	if (DECL_VINDEX (method) != NULL_TREE
+	    && host_integerp (DECL_VINDEX (method), 0))
+	  TREE_VEC_ELT (vtable, tree_low_cst (DECL_VINDEX (method), 0))
+	    = method;
     }
+
   return vtable;
 }
 
@@ -965,6 +1218,7 @@ static tree
 get_dispatch_table (type, this_class_addr)
      tree type, this_class_addr;
 {
+  int abstract_p = CLASS_ABSTRACT (TYPE_NAME (type));
   tree vtable = get_dispatch_vector (type);
   int i;
   tree list = NULL_TREE;
@@ -973,15 +1227,27 @@ get_dispatch_table (type, this_class_addr)
     {
       tree method = TREE_VEC_ELT (vtable, i);
       if (METHOD_ABSTRACT (method))
-	warning_with_decl (method, "abstract method in non-abstract class");
-      if (DECL_RTL (method) == 0)
-	make_decl_rtl (method, NULL, 1);
+	{
+	  if (! abstract_p)
+	    warning_with_decl (method,
+			       "abstract method in non-abstract class");
+	  method = null_pointer_node;
+	}
+      else
+	{
+	  if (DECL_RTL (method) == 0)
+	    make_decl_rtl (method, NULL, 1);
+	  method = build1 (ADDR_EXPR, nativecode_ptr_type_node, method);
+	}
       list = tree_cons (NULL_TREE /*DECL_VINDEX (method) + 2*/,
-			build1 (ADDR_EXPR, nativecode_ptr_type_node, method),
-			list);
+			method, list);
     }
-  /* Dummy entry for compatibility with G++ -fvtable-thunks. */
-  list = tree_cons (integer_zero_node, null_pointer_node, list);
+  /* Dummy entry for compatibility with G++ -fvtable-thunks.  When
+     using the Boehm GC we sometimes stash a GC type descriptor
+     there. We set the PURPOSE to NULL_TREE not to interfere (reset)
+     the emitted byte count during the output to the assembly file. */
+  list = tree_cons (NULL_TREE, get_boehm_type_descriptor (type),
+		    list);
   list = tree_cons (integer_zero_node, this_class_addr, list);
   return build (CONSTRUCTOR, build_prim_array_type (nativecode_ptr_type_node,
 						    nvirtuals + 2),
@@ -1070,6 +1336,7 @@ make_class_data (type)
     {
       tree init;
       if (METHOD_PRIVATE (method)
+	  && ! flag_keep_inline_functions
 	  && (flag_inline_functions || optimize))
 	continue;
       init = make_method_value (method);
@@ -1086,8 +1353,8 @@ make_class_data (type)
   DECL_IGNORED_P (methods_decl) = 1;
   rest_of_decl_compilation (methods_decl, (char*) 0, 1, 0);
 
-  if (flag_assume_compiled
-      && ! CLASS_ABSTRACT (type_decl) && ! CLASS_INTERFACE (type_decl))
+  if (assume_compiled (IDENTIFIER_POINTER (DECL_NAME (type_decl)))
+      && ! CLASS_INTERFACE (type_decl))
     {
       tree dtable = get_dispatch_table (type, this_class_addr);
       dtable_decl = build_dtable_decl (type);
@@ -1102,7 +1369,7 @@ make_class_data (type)
   super = CLASSTYPE_SUPER (type);
   if (super == NULL_TREE)
     super = null_pointer_node;
-  else if (flag_assume_compiled)
+  else if (assume_compiled (IDENTIFIER_POINTER (DECL_NAME (type_decl))))
     super = build_class_ref (super);
   else
     {
@@ -1128,7 +1395,7 @@ make_class_data (type)
 	  tree child = TREE_VEC_ELT (TYPE_BINFO_BASETYPES (type), i);
 	  tree iclass = BINFO_TYPE (child);
 	  tree index;
-	  if (flag_assume_compiled)
+	  if (assume_compiled (IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (iclass)))))
 	    index = build_class_ref (iclass);
 	  else
 	    {
@@ -1150,13 +1417,10 @@ make_class_data (type)
   constant_pool_constructor = build_constants_constructor ();
 
   START_RECORD_CONSTRUCTOR (temp, object_type_node);
-#if 0
-  PUSH_FIELD_VALUE (temp, "vtable", NULL_TREE);
-#else
   PUSH_FIELD_VALUE (temp, "vtable",
 		    build1 (ADDR_EXPR, dtable_ptr_type, class_dtable_decl));
-#endif
-  PUSH_FIELD_VALUE (temp, "sync_info", null_pointer_node);
+  if (! flag_hash_synchronization)
+    PUSH_FIELD_VALUE (temp, "sync_info", null_pointer_node);
   FINISH_RECORD_CONSTRUCTOR (temp);
   START_RECORD_CONSTRUCTOR (cons, class_type_node);
   PUSH_SUPER_VALUE (cons, temp);
@@ -1188,6 +1452,9 @@ make_class_data (type)
   PUSH_FIELD_VALUE (cons, "state", integer_zero_node);
 
   PUSH_FIELD_VALUE (cons, "thread", null_pointer_node);
+  PUSH_FIELD_VALUE (cons, "depth", integer_zero_node);
+  PUSH_FIELD_VALUE (cons, "ancestors", null_pointer_node);
+  PUSH_FIELD_VALUE (cons, "idt", null_pointer_node);
 
   FINISH_RECORD_CONSTRUCTOR (cons);
 
@@ -1196,29 +1463,49 @@ make_class_data (type)
 }
 
 void
-finish_class (cl)
-     tree cl;
+finish_class ()
 {
   tree method;
+  tree type_methods = TYPE_METHODS (CLASS_TO_HANDLE_TYPE (current_class));
+  int saw_native_method = 0;
 
-  /* Emit deferred inline methods. */
-  for ( method = TYPE_METHODS (CLASS_TO_HANDLE_TYPE (current_class));
-	method != NULL_TREE; method = TREE_CHAIN (method))
+  /* Find out if we have any native methods.  We use this information
+     later.  */
+  for (method = type_methods;
+       method != NULL_TREE;
+       method = TREE_CHAIN (method))
+    {
+      if (METHOD_NATIVE (method))
+	{
+	  saw_native_method = 1;
+	  break;
+	}
+    }
+
+  /* Emit deferred inline methods. */  
+  for (method = type_methods; method != NULL_TREE; )
     {
       if (! TREE_ASM_WRITTEN (method) && DECL_SAVED_INSNS (method) != 0)
 	{
 	  /* It's a deferred inline method.  Decide if we need to emit it. */
 	  if (flag_keep_inline_functions
 	      || TREE_SYMBOL_REFERENCED (DECL_ASSEMBLER_NAME (method))
-	      || ! METHOD_PRIVATE (method))
+	      || ! METHOD_PRIVATE (method)
+	      || saw_native_method)
 	    {
 	      temporary_allocation ();
 	      output_inline_function (method);
 	      permanent_allocation (1);
+	      /* Scan the list again to see if there are any earlier
+                 methods to emit. */
+	      method = type_methods;
+	      continue;
 	    }
 	}
+      method = TREE_CHAIN (method);
     }
 
+  current_function_decl = NULL_TREE;
   make_class_data (current_class);
   register_class ();
   rest_of_decl_compilation (TYPE_NAME (current_class), (char*) 0, 1, 0);
@@ -1242,8 +1529,7 @@ is_compiled_class (class)
   if (class == current_class)
     return 2;
 
-  seen_in_zip = (TYPE_LANG_SPECIFIC (class) && TYPE_LANG_SPECIFIC (class)->jcf
-		 && TYPE_LANG_SPECIFIC (class)->jcf->seen_in_zip);
+  seen_in_zip = (TYPE_JCF (class) && TYPE_JCF (class)->seen_in_zip);
   if (CLASS_FROM_CURRENTLY_COMPILED_SOURCE_P (class) || seen_in_zip)
     {
       /* The class was seen in the current ZIP file and will be
@@ -1259,7 +1545,7 @@ is_compiled_class (class)
       return 2;
     }
 
-  if (flag_assume_compiled)
+  if (assume_compiled (IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (class)))))
     {
       if (!CLASS_LOADED_P (class))
 	{
@@ -1324,7 +1610,7 @@ append_gpp_mangled_type (obstack, type)
 	}
       else
 	{
-	  char *class_name = IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (type)));
+	  const char *class_name = IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (type)));
 	  append_gpp_mangled_classtype (obstack, class_name);
 	}
       break;
@@ -1403,13 +1689,33 @@ tree
 build_dtable_decl (type)
      tree type;
 {
-  tree name;
+  tree name, dtype;
+
+  /* We need to build a new dtable type so that its size is uniquely
+     computed when we're dealing with the class for real and not just
+     faking it (like java.lang.Class during the initialization of the
+     compiler.) We now we're not faking a class when CURRENT_CLASS is
+     TYPE. */
+  if (current_class == type)
+    {
+      tree dummy, aomt, n;
+
+      dtype = make_node (RECORD_TYPE);
+      PUSH_FIELD (dtype, dummy, "class", class_ptr_type);
+      n = build_int_2 (TREE_VEC_LENGTH (get_dispatch_vector (type)), 0);
+      aomt = build_array_type (ptr_type_node, build_index_type (n));
+      PUSH_FIELD (dtype, dummy, "methods", aomt);
+      layout_type (dtype);
+    }
+  else
+    dtype = dtable_type;
+
   obstack_grow (&temporary_obstack, "__vt_", 5);
   append_gpp_mangled_type (&temporary_obstack, type);
   obstack_1grow (&temporary_obstack, '\0');
   name = get_identifier (obstack_base (&temporary_obstack));
   obstack_free (&temporary_obstack, obstack_base (&temporary_obstack));
-  return build_decl (VAR_DECL, name, dtable_type);
+  return build_decl (VAR_DECL, name, dtype);
 }
 
 /* Pre-pend the TYPE_FIELDS of THIS_CLASS with a dummy FIELD_DECL for the
@@ -1420,6 +1726,9 @@ push_super_field (this_class, super_class)
      tree this_class, super_class;
 {
   tree base_decl;
+  /* Don't insert the field if we're just re-laying the class out. */ 
+  if (TYPE_FIELDS (this_class) && !DECL_NAME (TYPE_FIELDS (this_class)))
+    return;
   push_obstacks (&permanent_obstack, &permanent_obstack);
   base_decl = build_decl (FIELD_DECL, NULL_TREE, super_class);
   pop_obstacks ();
@@ -1427,6 +1736,7 @@ push_super_field (this_class, super_class)
   TREE_CHAIN (base_decl) = TYPE_FIELDS (this_class);
   TYPE_FIELDS (this_class) = base_decl;
   DECL_SIZE (base_decl) = TYPE_SIZE (super_class);
+  DECL_SIZE_UNIT (base_decl) = TYPE_SIZE_UNIT (super_class);
 }
 
 /* Handle the different manners we may have to lay out a super class.  */
@@ -1452,7 +1762,8 @@ maybe_layout_super_class (super_class, this_class)
 	super_class = TREE_TYPE (super_class);
       else
 	{
-	  super_class = do_resolve_class (super_class, NULL_TREE, this_class);
+	  super_class = do_resolve_class (NULL_TREE, /* FIXME? */
+					  super_class, NULL_TREE, this_class);
 	  if (!super_class)
 	    return NULL_TREE;	/* FIXME, NULL_TREE not checked by caller. */
 	  super_class = TREE_TYPE (super_class);
@@ -1468,8 +1779,36 @@ void
 layout_class (this_class)
      tree this_class;
 {
+  static tree list = NULL_TREE;
   tree super_class = CLASSTYPE_SUPER (this_class);
   tree field;
+  
+  list = tree_cons (this_class, NULL_TREE, list);
+  if (CLASS_BEING_LAIDOUT (this_class))
+    {
+      char buffer [1024];
+      tree current;
+      
+      sprintf (buffer, " with `%s'",
+	       IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (this_class))));
+      obstack_grow (&temporary_obstack, buffer, strlen (buffer));
+
+      for (current = TREE_CHAIN (list); current; 
+	   current = TREE_CHAIN (current))
+	{
+	  tree decl = TYPE_NAME (TREE_PURPOSE (current));
+	  sprintf (buffer, "\n  which inherits from `%s' (%s:%d)",
+		   IDENTIFIER_POINTER (DECL_NAME (decl)),
+		   DECL_SOURCE_FILE (decl),
+		   DECL_SOURCE_LINE (decl));
+	  obstack_grow (&temporary_obstack, buffer, strlen (buffer));
+	}
+      obstack_1grow (&temporary_obstack, '\0');
+      cyclic_inheritance_report = obstack_finish (&temporary_obstack);
+      TYPE_SIZE (this_class) = error_mark_node;
+      return;
+    }
+  CLASS_BEING_LAIDOUT (this_class) = 1;
 
   if (super_class)
     {
@@ -1477,6 +1816,8 @@ layout_class (this_class)
       if (TREE_CODE (TYPE_SIZE (super_class)) == ERROR_MARK)
 	{
 	  TYPE_SIZE (this_class) = error_mark_node;
+	  CLASS_BEING_LAIDOUT (this_class) = 0;
+	  list = TREE_CHAIN (list);
 	  return;
 	}
       if (TYPE_SIZE (this_class) == NULL_TREE)
@@ -1494,6 +1835,13 @@ layout_class (this_class)
     }
 
   layout_type (this_class);
+
+  /* Convert the size back to an SI integer value */
+  TYPE_SIZE_UNIT (this_class) = 
+    fold (convert (int_type_node, TYPE_SIZE_UNIT (this_class)));
+
+  CLASS_BEING_LAIDOUT (this_class) = 0;
+  list = TREE_CHAIN (list);
 }
 
 void
@@ -1542,7 +1890,7 @@ tree
 layout_class_method (this_class, super_class, method_decl, dtable_count)
      tree this_class, super_class, method_decl, dtable_count;
 {
-  char *ptr;
+  const char *ptr;
   char *asm_name;
   tree arg, arglist, t;
   int method_name_needs_escapes = 0;
@@ -1552,8 +1900,7 @@ layout_class_method (this_class, super_class, method_decl, dtable_count)
   if (method_name_is_wfl)
     method_name = java_get_real_method_name (method_decl);
 
-  if (method_name != init_identifier_node 
-      && method_name != finit_identifier_node)
+  if (!ID_INIT_P (method_name) && !ID_FINIT_P (method_name))
     {
       int encoded_len
 	= unicode_mangling_length (IDENTIFIER_POINTER (method_name), 
@@ -1574,7 +1921,7 @@ layout_class_method (this_class, super_class, method_decl, dtable_count)
     }
       
   obstack_grow (&temporary_obstack, "__", 2);
-  if (method_name == finit_identifier_node)
+  if (ID_FINIT_P (method_name))
     obstack_grow (&temporary_obstack, "finit", 5);
   append_gpp_mangled_type (&temporary_obstack, this_class);
   TREE_PUBLIC (method_decl) = 1;
@@ -1636,13 +1983,13 @@ layout_class_method (this_class, super_class, method_decl, dtable_count)
      it's an interface method that isn't clinit. */
   if (! METHOD_ABSTRACT (method_decl) 
       || (CLASS_INTERFACE (TYPE_NAME (this_class)) 
-	  && (IS_CLINIT (method_decl))))
+	  && (DECL_CLINIT_P (method_decl))))
     make_function_rtl (method_decl);
   obstack_free (&temporary_obstack, asm_name);
 
-  if (method_name == init_identifier_node)
+  if (ID_INIT_P (method_name))
     {
-      char *p = IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (this_class)));
+      const char *p = IDENTIFIER_POINTER (DECL_NAME (TYPE_NAME (this_class)));
       for (ptr = p; *ptr; )
 	{
 	  if (*ptr++ == '.')
@@ -1685,9 +2032,11 @@ layout_class_method (this_class, super_class, method_decl, dtable_count)
 	       && dtable_count)
 	{
 	  DECL_VINDEX (method_decl) = dtable_count;
-	  dtable_count = build_int_2 (1+TREE_INT_CST_LOW (dtable_count), 0);
+	  dtable_count = fold (build (PLUS_EXPR, integer_type_node,
+				      dtable_count, integer_one_node));
 	}
     }
+
   return dtable_count;
 }
 
@@ -1715,13 +2064,12 @@ register_class ()
 void
 emit_register_classes ()
 {
-  extern tree get_file_function_name PROTO((int));
+  extern tree get_file_function_name PARAMS ((int));
   tree init_name = get_file_function_name ('I');
   tree init_type = build_function_type (void_type_node, end_params_node);
   tree init_decl;
   tree t;
 
-  start_sequence ();
   init_decl = build_decl (FUNCTION_DECL, init_name, init_type);
   DECL_ASSEMBLER_NAME (init_decl) = init_name;
   TREE_STATIC (init_decl) = 1;
