@@ -1,7 +1,6 @@
 /* Process declarations and variables for the GNU compiler for the
    Java(TM) language.
-
-   Copyright (C) 1996, 97-98, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
 
 This file is part of GNU CC.
 
@@ -29,15 +28,31 @@ The Free Software Foundation is independent of Sun Microsystems, Inc.  */
 #include "config.h"
 #include "system.h"
 #include "tree.h"
+#include "toplev.h"
+#include "flags.h"
 #include "java-tree.h"
 #include "jcf.h"
 #include "toplev.h"
+#include "function.h"
 #include "except.h"
+#include "defaults.h"
+#include "java-except.h"
 
-static tree push_jvm_slot PROTO ((int, tree));
-static tree builtin_function PROTO ((const char *, tree,
-				     enum built_in_function, const char *));
-static tree lookup_name_current_level PROTO ((tree));
+#if defined (DEBUG_JAVA_BINDING_LEVELS)
+extern void indent PROTO((void));
+#endif
+
+static tree push_jvm_slot PARAMS ((int, tree));
+static tree lookup_name_current_level PARAMS ((tree));
+static tree push_promoted_type PARAMS ((const char *, tree));
+static struct binding_level *make_binding_level PARAMS ((void));
+static boolean emit_init_test_initialization PARAMS ((struct hash_entry *,
+						      hash_table_key));
+static tree create_primitive_vtable PARAMS ((const char *));
+
+/* Set to non-zero value in order to emit class initilization code
+   before static field references.  */
+extern int always_initialize_class_p;
 
 #ifndef INT_TYPE_SIZE
 #define INT_TYPE_SIZE BITS_PER_WORD
@@ -61,6 +76,21 @@ tree pending_local_decls = NULL_TREE;
 /* Push a local variable or stack slot into the decl_map,
    and assign it an rtl. */
 
+#if defined(DEBUG_JAVA_BINDING_LEVELS)
+int binding_depth = 0;
+int is_class_level = 0;
+int current_pc;
+
+void
+indent ()
+{
+  register unsigned i;
+
+  for (i = 0; i < binding_depth*2; i++)
+    putc (' ', stderr);
+}
+#endif /* defined(DEBUG_JAVA_BINDING_LEVELS) */
+
 static tree
 push_jvm_slot (index, decl)
      int index;
@@ -109,9 +139,7 @@ push_jvm_slot (index, decl)
 
 /* Find a VAR_DECL (or PARM_DECL) at local index INDEX that has type TYPE,
    that is valid at PC (or -1 if any pc).
-   If there is no existing matching decl, allocate one.
-   If we find a decl with matching modes but different types,
-   we re-use the rtl, but create a new decl. */
+   If there is no existing matching decl, allocate one.  */
 
 tree
 find_local_variable (index, type, pc)
@@ -121,6 +149,7 @@ find_local_variable (index, type, pc)
 {
   tree decl = TREE_VEC_ELT (decl_map, index);
   tree best = NULL_TREE;
+
   while (decl != NULL_TREE)
     {
       int in_range;
@@ -129,6 +158,10 @@ find_local_variable (index, type, pc)
 	    && pc < DECL_LOCAL_END_PC (decl));
 
       if ((TREE_TYPE (decl) == type
+	   || (TREE_CODE (TREE_TYPE (decl)) == TREE_CODE (type)
+	       && TYPE_PRECISION (TREE_TYPE (decl)) <= 32
+	       && TYPE_PRECISION (type) <= 32
+	       && TREE_CODE (type) != POINTER_TYPE)
 	   || (TREE_CODE (TREE_TYPE (decl)) == POINTER_TYPE
 	       && type == ptr_type_node))
 	   && in_range)
@@ -197,6 +230,13 @@ struct binding_level
 
     /* The bytecode PC that marks the end of this level. */
     int end_pc;
+    /* The bytecode PC that marks the start of this level. */
+    int start_pc;
+
+#if defined(DEBUG_JAVA_BINDING_LEVELS)
+    /* Binding depth at which this level began.  */
+    unsigned binding_depth;
+#endif /* defined(DEBUG_JAVA_BINDING_LEVELS) */
   };
 
 #define NULL_BINDING_LEVEL (struct binding_level *) NULL
@@ -215,11 +255,15 @@ static struct binding_level *free_binding_level;
 
 static struct binding_level *global_binding_level;
 
+/* A PC value bigger than any PC value we may ever may encounter. */
+
+#define LARGEST_PC (( (unsigned int)1 << (HOST_BITS_PER_INT - 1)) - 1)
+
 /* Binding level structures are initialized by copying this one.  */
 
 static struct binding_level clear_binding_level
   = {NULL_TREE, NULL_TREE, NULL_TREE, NULL_TREE,
-       NULL_BINDING_LEVEL, 0, 0, 0, 0, 1000000000};
+       NULL_BINDING_LEVEL, 0, 0, 0, 0, LARGEST_PC, 0};
 
 #if 0
 /* A list (chain of TREE_LIST nodes) of all LABEL_DECLs in the function
@@ -244,13 +288,6 @@ static int keep_next_level_flag;
 
 static int keep_next_if_subblocks;
   
-/* The FUNCTION_DECL for the function currently being compiled,
-   or 0 if between functions.  */
-tree current_function_decl;
-
-/* The type node for the ordinary character type.  */
-tree char_type_node;
-
 tree object_type_node;
 tree unqualified_object_id_node;
 tree object_ptr_type_node;
@@ -259,22 +296,14 @@ tree string_ptr_type_node;
 tree throwable_type_node;
 tree runtime_exception_type_node;
 tree error_exception_type_node;
+tree rawdata_ptr_type_node;
 tree *predef_filenames;
 int  predef_filenames_size;
 
 tree boolean_type_node;
 
-tree float_type_node;
-tree double_type_node;
-
-/* a VOID_TYPE node.  */
-
-tree void_type_node;
-tree ptr_type_node;
 tree return_address_type_node;
 
-tree integer_type_node;
-
 tree byte_type_node;
 tree short_type_node;
 tree int_type_node;
@@ -315,19 +344,8 @@ tree one_elt_array_domain_type;
 tree access_flags_type_node;
 tree class_dtable_decl;
 
-/* a node which has tree code ERROR_MARK, and whose type is itself.
-   All erroneous expressions are replaced with this node.  All functions
-   that accept nodes as arguments should avoid generating error messages
-   if this node is one of the arguments, since it is undesirable to get
-   multiple error messages from one error in the input.  */
-
-tree error_mark_node;
-
-/* Two expressions that are constants with value zero.
-   The first is of type `int', the second of type `void *'.
-   Other of type `long', `float' and `double' follow.  */
-tree integer_zero_node;
-tree null_pointer_node;
+/* Expressions that are constants with value zero, of types
+   `long', `float' and `double'.  */
 tree long_zero_node;
 tree float_zero_node;
 tree double_zero_node;
@@ -335,7 +353,8 @@ tree double_zero_node;
 tree empty_stmt_node;
 
 /* Nodes for boolean constants TRUE and FALSE. */
-tree boolean_true_node, boolean_false_node;
+tree boolean_true_node;
+tree boolean_false_node;
 
 tree TYPE_identifier_node;
 tree init_identifier_node;
@@ -346,7 +365,7 @@ tree length_identifier_node;
 tree this_identifier_node;
 tree super_identifier_node;
 tree continue_identifier_node;
-
+tree access0_identifier_node;	/* 1.1 */
 tree end_params_node;
 
 /* References to internal libjava functions we use. */
@@ -358,20 +377,38 @@ tree soft_newarray_node;
 tree soft_anewarray_node;
 tree soft_multianewarray_node;
 tree soft_badarrayindex_node;
-tree throw_node;
+tree soft_nullpointer_node;
+tree throw_node [2];
 tree soft_checkarraystore_node;
 tree soft_monitorenter_node;
 tree soft_monitorexit_node;
 tree soft_lookupinterfacemethod_node;
+tree soft_lookupjnimethod_node;
+tree soft_getjnienvnewframe_node;
+tree soft_jnipopsystemframe_node;
 tree soft_fmod_node;
 tree soft_exceptioninfo_call_node;
+tree soft_idiv_node;
+tree soft_irem_node;
+tree soft_ldiv_node;
+tree soft_lrem_node;
+
+/* Declarations for vtables for primitive arrays.  */
+tree boolean_array_vtable;
+tree byte_array_vtable;
+tree char_array_vtable;
+tree short_array_vtable;
+tree int_array_vtable;
+tree long_array_vtable;
+tree float_array_vtable;
+tree double_array_vtable;
 
 /* Build (and pushdecl) a "promoted type" for all standard
    types shorter than int.  */
 
 static tree
 push_promoted_type (name, actual_type)
-     char *name;
+     const char *name;
      tree actual_type;
 {
   tree type = make_node (TREE_CODE (actual_type));
@@ -382,11 +419,9 @@ push_promoted_type (name, actual_type)
   tree in_min = TYPE_MIN_VALUE (actual_type);
   tree in_max = TYPE_MAX_VALUE (actual_type);
 #endif
-  TYPE_MIN_VALUE (type) = build_int_2 (TREE_INT_CST_LOW (in_min),
-				       TREE_INT_CST_HIGH (in_min));
+  TYPE_MIN_VALUE (type) = copy_node (in_min);
   TREE_TYPE (TYPE_MIN_VALUE (type)) = type;
-  TYPE_MAX_VALUE (type) = build_int_2 (TREE_INT_CST_LOW (in_max),
-				       TREE_INT_CST_HIGH (in_max));
+  TYPE_MAX_VALUE (type) = copy_node (in_max);
   TREE_TYPE (TYPE_MAX_VALUE (type)) = type;
   TYPE_PRECISION (type) = TYPE_PRECISION (int_type_node);
   layout_type (type);
@@ -395,7 +430,8 @@ push_promoted_type (name, actual_type)
 }
 
 /* Nodes for integer constants.  */
-tree integer_one_node, integer_two_node, integer_four_node;
+tree integer_two_node;
+tree integer_four_node;
 tree integer_negative_one_node;
 
 /* Return a definition for a builtin function named NAME and whose data type
@@ -406,11 +442,12 @@ tree integer_negative_one_node;
    If LIBRARY_NAME is nonzero, use that for DECL_ASSEMBLER_NAME,
    the name to be called if we can't opencode the function.  */
 
-static tree
-builtin_function (name, type, function_code, library_name)
+tree
+builtin_function (name, type, function_code, class, library_name)
      const char *name;
      tree type;
-     enum built_in_function function_code;
+     int function_code;
+     enum built_in_class class;
      const char *library_name;
 {
   tree decl = build_decl (FUNCTION_DECL, get_identifier (name), type);
@@ -420,14 +457,26 @@ builtin_function (name, type, function_code, library_name)
     DECL_ASSEMBLER_NAME (decl) = get_identifier (library_name);
   make_decl_rtl (decl, NULL_PTR, 1);
   pushdecl (decl);
-  if (function_code != NOT_BUILT_IN)
-    {
-      DECL_BUILT_IN (decl) = 1;
-      DECL_FUNCTION_CODE (decl) = function_code;
-    }
+  DECL_BUILT_IN_CLASS (decl) = class;
+  DECL_FUNCTION_CODE (decl) = function_code;
   return decl;
 }
 
+/* Return tree that represents a vtable for a primitive array.  */
+static tree
+create_primitive_vtable (name)
+     const char *name;
+{
+  tree r;
+  char buf[50];
+
+  sprintf (buf, "_Jv_%sVTable", name);
+  r = build_decl (VAR_DECL, get_identifier (buf), ptr_type_node);
+  DECL_EXTERNAL (r) = 1;
+  make_decl_rtl (r, buf, 1);
+  return r;
+}
+
 void
 init_decl_processing ()
 {
@@ -441,15 +490,13 @@ init_decl_processing ()
   pushlevel (0);	/* make the binding_level structure for global names */
   global_binding_level = current_binding_level;
 
+  /* The code here must be similar to build_common_tree_nodes{,_2} in
+     tree.c, especially as to the order of initializing common nodes.  */
   error_mark_node = make_node (ERROR_MARK);
   TREE_TYPE (error_mark_node) = error_mark_node;
 
   /* Create sizetype first - needed for other types. */
-  sizetype = make_unsigned_type (POINTER_SIZE);
-  size_zero_node = build_int_2 (0, 0);
-  TREE_TYPE (size_zero_node) = sizetype;
-  size_one_node = build_int_2 (1, 0);
-  TREE_TYPE (size_one_node) = sizetype;
+  initialize_sizetypes ();
 
   byte_type_node = make_signed_type (8);
   pushdecl (build_decl (TYPE_DECL, get_identifier ("byte"), byte_type_node));
@@ -473,14 +520,22 @@ init_decl_processing ()
   pushdecl (build_decl (TYPE_DECL, get_identifier ("unsigned long"),
 			unsigned_long_type_node));
 
-  integer_type_node = type_for_size (INT_TYPE_SIZE, 0);
+  set_sizetype (make_unsigned_type (POINTER_SIZE));
 
+  /* Define these next since types below may used them.  */
+  integer_type_node = type_for_size (INT_TYPE_SIZE, 0);
   integer_zero_node = build_int_2 (0, 0);
   integer_one_node = build_int_2 (1, 0);
   integer_two_node = build_int_2 (2, 0);
   integer_four_node = build_int_2 (4, 0);
   integer_negative_one_node = build_int_2 (-1, 0);
 
+  size_zero_node = size_int (0);
+  size_one_node = size_int (1);
+  bitsize_zero_node = bitsize_int (0);
+  bitsize_one_node = bitsize_int (1);
+  bitsize_unit_node = bitsize_int (BITS_PER_UNIT);
+
   long_zero_node = build_int_2 (0, 0);
   TREE_TYPE (long_zero_node) = long_type_node;
 
@@ -544,7 +599,17 @@ init_decl_processing ()
   float_zero_node = build_real (float_type_node, dconst0);
   double_zero_node = build_real (double_type_node, dconst0);
 
-  /* As your adding items here, please update the code right after
+  /* These are the vtables for arrays of primitives.  */
+  boolean_array_vtable = create_primitive_vtable ("boolean");
+  byte_array_vtable = create_primitive_vtable ("byte");
+  char_array_vtable = create_primitive_vtable ("char");
+  short_array_vtable = create_primitive_vtable ("short");
+  int_array_vtable = create_primitive_vtable ("int");
+  long_array_vtable = create_primitive_vtable ("long");
+  float_array_vtable = create_primitive_vtable ("float");
+  double_array_vtable = create_primitive_vtable ("double");
+
+  /* As you're adding items here, please update the code right after
      this section, so that the filename containing the source code of
      the pre-defined class gets registered correctly. */
   unqualified_object_id_node = get_identifier ("Object");
@@ -558,10 +623,12 @@ init_decl_processing ()
     lookup_class (get_identifier ("java.lang.RuntimeException"));
   error_exception_type_node = 
     lookup_class (get_identifier ("java.lang.Error"));
+  rawdata_ptr_type_node
+    = promote_type (lookup_class (get_identifier ("gnu.gcj.RawData")));
 
   /* This section has to be updated as items are added to the previous
      section. */
-  predef_filenames_size = 6;
+  predef_filenames_size = 7;
   predef_filenames = (tree *)xmalloc (predef_filenames_size * sizeof (tree));
   predef_filenames [0] = get_identifier ("java/lang/Class.java");
   predef_filenames [1] = get_identifier ("java/lang/Error.java");
@@ -569,6 +636,7 @@ init_decl_processing ()
   predef_filenames [3] = get_identifier ("java/lang/RuntimeException.java");
   predef_filenames [4] = get_identifier ("java/lang/String.java");
   predef_filenames [5] = get_identifier ("java/lang/Throwable.java");
+  predef_filenames [6] = get_identifier ("gnu/gcj/RawData.java");
 
   methodtable_type = make_node (RECORD_TYPE);
   layout_type (methodtable_type);
@@ -584,6 +652,7 @@ init_decl_processing ()
   this_identifier_node = get_identifier ("this");
   super_identifier_node = get_identifier ("super");
   continue_identifier_node = get_identifier ("continue");
+  access0_identifier_node = get_identifier ("access$0");
 
   /* for lack of a better place to put this stub call */
   init_expr_processing();
@@ -607,7 +676,12 @@ init_decl_processing ()
   dtable_ptr_type = build_pointer_type (dtable_type);
 
   PUSH_FIELD (object_type_node, field, "vtable", dtable_ptr_type);
-  PUSH_FIELD (object_type_node, field, "sync_info", ptr_type_node);
+  /* This isn't exactly true, but it is what we have in the source.
+     There is an unresolved issue here, which is whether the vtable
+     should be marked by the GC.  */
+  if (! flag_hash_synchronization)
+    PUSH_FIELD (object_type_node, field, "sync_info",
+		build_pointer_type (object_type_node));
   for (t = TYPE_FIELDS (object_type_node); t != NULL_TREE; t = TREE_CHAIN (t))
     FIELD_PRIVATE (t) = 1;
   FINISH_RECORD (object_type_node);
@@ -646,6 +720,9 @@ init_decl_processing ()
   PUSH_FIELD (class_type_node, field, "interface_count", short_type_node);
   PUSH_FIELD (class_type_node, field, "state", byte_type_node);
   PUSH_FIELD (class_type_node, field, "thread", ptr_type_node);
+  PUSH_FIELD (class_type_node, field, "depth", short_type_node);
+  PUSH_FIELD (class_type_node, field, "ancestors", ptr_type_node);
+  PUSH_FIELD (class_type_node, field, "idt", ptr_type_node);  
   for (t = TYPE_FIELDS (class_type_node);  t != NULL_TREE;  t = TREE_CHAIN (t))
     FIELD_PRIVATE (t) = 1;
   push_super_field (class_type_node, object_type_node);
@@ -717,27 +794,42 @@ init_decl_processing ()
 		 tree_cons (NULL_TREE, int_type_node, endlink));
   alloc_object_node = builtin_function ("_Jv_AllocObject",
 					build_function_type (ptr_type_node, t),
-					NOT_BUILT_IN, NULL_PTR);
+					0, NOT_BUILT_IN, NULL_PTR);
+  DECL_IS_MALLOC (alloc_object_node) = 1;
+
+  t = tree_cons (NULL_TREE, ptr_type_node, endlink);
   soft_initclass_node = builtin_function ("_Jv_InitClass",
 					  build_function_type (void_type_node,
 							       t),
-					  NOT_BUILT_IN, NULL_PTR);
+					  0, NOT_BUILT_IN,
+					  NULL_PTR);
+  throw_node[0] = builtin_function ("_Jv_Throw",
+				    build_function_type (ptr_type_node, t),
+				    0, NOT_BUILT_IN, NULL_PTR);
+  /* Mark throw_nodes as `noreturn' functions with side effects.  */
+  TREE_THIS_VOLATILE (throw_node[0]) = 1;
+  TREE_SIDE_EFFECTS (throw_node[0]) = 1;
   t = tree_cons (NULL_TREE, ptr_type_node, endlink);
-  throw_node = builtin_function ("_Jv_Throw",
-				 build_function_type (ptr_type_node, t),
-				 NOT_BUILT_IN, NULL_PTR);
+  throw_node[1] = builtin_function ("_Jv_Sjlj_Throw",
+				    build_function_type (ptr_type_node, t),
+				    0, NOT_BUILT_IN, NULL_PTR);
+  TREE_THIS_VOLATILE (throw_node[1]) = 1;
+  TREE_SIDE_EFFECTS (throw_node[1]) = 1;
   t = build_function_type (int_type_node, endlink);
   soft_monitorenter_node 
-      = builtin_function ("_Jv_MonitorEnter", t, NOT_BUILT_IN, NULL_PTR);
+    = builtin_function ("_Jv_MonitorEnter", t, 0, NOT_BUILT_IN,
+			NULL_PTR);
   soft_monitorexit_node 
-      = builtin_function ("_Jv_MonitorExit", t, NOT_BUILT_IN, NULL_PTR);
+    = builtin_function ("_Jv_MonitorExit", t, 0, NOT_BUILT_IN,
+			NULL_PTR);
   
   t = tree_cons (NULL_TREE, int_type_node, 
 		 tree_cons (NULL_TREE, int_type_node, endlink));
   soft_newarray_node
       = builtin_function ("_Jv_NewArray",
 			  build_function_type(ptr_type_node, t),
-			  NOT_BUILT_IN, NULL_PTR );
+			  0, NOT_BUILT_IN, NULL_PTR);
+  DECL_IS_MALLOC (soft_newarray_node) = 1;
 
   t = tree_cons (NULL_TREE, int_type_node,
 		 tree_cons (NULL_TREE, class_ptr_type,
@@ -745,54 +837,85 @@ init_decl_processing ()
   soft_anewarray_node
       = builtin_function ("_Jv_NewObjectArray",
 			  build_function_type (ptr_type_node, t),
-			  NOT_BUILT_IN, NULL_PTR );
+			  0, NOT_BUILT_IN, NULL_PTR);
+  DECL_IS_MALLOC (soft_anewarray_node) = 1;
 
   t = tree_cons (NULL_TREE, ptr_type_node,
 		 tree_cons (NULL_TREE, int_type_node, endlink));
   soft_multianewarray_node
       = builtin_function ("_Jv_NewMultiArray",
 			  build_function_type (ptr_type_node, t),
-			  NOT_BUILT_IN, NULL_PTR );
+			  0, NOT_BUILT_IN, NULL_PTR);
+  DECL_IS_MALLOC (soft_multianewarray_node) = 1;
 
   t = build_function_type (void_type_node, 
 			   tree_cons (NULL_TREE, int_type_node, endlink));
   soft_badarrayindex_node
       = builtin_function ("_Jv_ThrowBadArrayIndex", t, 
-			  NOT_BUILT_IN, NULL_PTR);
+			  0, NOT_BUILT_IN, NULL_PTR);
+  /* Mark soft_badarrayindex_node as a `noreturn' function with side
+     effects.  */
   TREE_THIS_VOLATILE (soft_badarrayindex_node) = 1;
   TREE_SIDE_EFFECTS (soft_badarrayindex_node) = 1;
 
+  soft_nullpointer_node
+    = builtin_function ("_Jv_ThrowNullPointerException",
+			build_function_type (void_type_node, endlink),
+			0, NOT_BUILT_IN, NULL_PTR);
+  /* Mark soft_nullpointer_node as a `noreturn' function with side
+     effects.  */
+  TREE_THIS_VOLATILE (soft_nullpointer_node) = 1;
+  TREE_SIDE_EFFECTS (soft_nullpointer_node) = 1;
+
   t = tree_cons (NULL_TREE, class_ptr_type,
 		 tree_cons (NULL_TREE, object_ptr_type_node, endlink));
   soft_checkcast_node
     = builtin_function ("_Jv_CheckCast",
 			build_function_type (ptr_type_node, t),
-			NOT_BUILT_IN, NULL_PTR);
+			0, NOT_BUILT_IN, NULL_PTR);
   t = tree_cons (NULL_TREE, object_ptr_type_node,
 		 tree_cons (NULL_TREE, class_ptr_type, endlink));
   soft_instanceof_node
     = builtin_function ("_Jv_IsInstanceOf",
-			build_function_type (promoted_boolean_type_node, t),
-			NOT_BUILT_IN, NULL_PTR);
+			build_function_type (boolean_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
   t = tree_cons (NULL_TREE, object_ptr_type_node,
 		 tree_cons (NULL_TREE, object_ptr_type_node, endlink));
   soft_checkarraystore_node
     = builtin_function ("_Jv_CheckArrayStore",
 			build_function_type (void_type_node, t),
-			NOT_BUILT_IN, NULL_PTR);
+			0, NOT_BUILT_IN, NULL_PTR);
   t = tree_cons (NULL_TREE, ptr_type_node,
 		 tree_cons (NULL_TREE, ptr_type_node,
-			    tree_cons (NULL_TREE, ptr_type_node, endlink)));
+			    tree_cons (NULL_TREE, int_type_node, endlink)));
   soft_lookupinterfacemethod_node 
-    = builtin_function ("_Jv_LookupInterfaceMethod",
+    = builtin_function ("_Jv_LookupInterfaceMethodIdx",
 			build_function_type (ptr_type_node, t),
-			NOT_BUILT_IN, NULL_PTR);
+			0, NOT_BUILT_IN, NULL_PTR);
+
+  t = tree_cons (NULL_TREE, object_ptr_type_node,
+		 tree_cons (NULL_TREE, ptr_type_node,
+			    tree_cons (NULL_TREE, ptr_type_node, endlink)));
+  soft_lookupjnimethod_node
+    = builtin_function ("_Jv_LookupJNIMethod",
+			build_function_type (ptr_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
+  t = tree_cons (NULL_TREE, ptr_type_node, endlink);
+  soft_getjnienvnewframe_node
+    = builtin_function ("_Jv_GetJNIEnvNewFrame",
+			build_function_type (ptr_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
+  soft_jnipopsystemframe_node
+    = builtin_function ("_Jv_JNI_PopSystemFrame",
+			build_function_type (ptr_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
+
   t = tree_cons (NULL_TREE, double_type_node,
 		 tree_cons (NULL_TREE, double_type_node, endlink));
   soft_fmod_node
     = builtin_function ("__builtin_fmod",
 			build_function_type (double_type_node, t),
-			BUILT_IN_FMOD, "fmod");
+			BUILT_IN_FMOD, BUILT_IN_NORMAL, "fmod");
 
   soft_exceptioninfo_call_node
     = build (CALL_EXPR, 
@@ -800,7 +923,7 @@ init_decl_processing ()
 	     build_address_of 
 	       (builtin_function ("_Jv_exception_info", 
 				  build_function_type (ptr_type_node, endlink),
-				  NOT_BUILT_IN, NULL_PTR)),
+				  0, NOT_BUILT_IN, NULL_PTR)),
 	     NULL_TREE, NULL_TREE);
   TREE_SIDE_EFFECTS (soft_exceptioninfo_call_node) = 1;
 #if 0
@@ -809,9 +932,29 @@ init_decl_processing ()
   soft_fmodf_node
     = builtin_function ("__builtin_fmodf",
 			build_function_type (float_type_node, t),
-			BUILT_IN_FMOD, "fmodf");
+			BUILT_IN_FMOD, BUILT_IN_NORMAL, "fmodf");
 #endif
     
+  soft_idiv_node
+    = builtin_function ("_Jv_divI",
+			build_function_type (int_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
+
+  soft_irem_node
+    = builtin_function ("_Jv_remI",
+			build_function_type (int_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
+
+  soft_ldiv_node
+    = builtin_function ("_Jv_divJ",
+			build_function_type (long_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
+
+  soft_lrem_node
+    = builtin_function ("_Jv_remJ",
+			build_function_type (long_type_node, t),
+			0, NOT_BUILT_IN, NULL_PTR);
+
   init_class_processing ();
 }
 
@@ -907,11 +1050,12 @@ pushdecl (x)
   register tree t;
   register tree name = DECL_NAME (x);
   register struct binding_level *b = current_binding_level;
-
-  DECL_CONTEXT (x) = current_function_decl;
+  
+  if (TREE_CODE (x) != TYPE_DECL)
+    DECL_CONTEXT (x) = current_function_decl;
   if (name)
     {
-      char *file;
+      const char *file;
       int line;
 
       t = lookup_name_current_level (name);
@@ -980,7 +1124,7 @@ pushdecl (x)
 		   /* No shadow warnings for vars made for inlining.  */
 		   && ! DECL_FROM_INLINE (x))
 	    {
-	      char *warnstring = 0;
+	      const char *warnstring = 0;
 
 	      if (TREE_CODE (x) == PARM_DECL
 		  && current_binding_level->level_chain->parm_flag)
@@ -1017,6 +1161,7 @@ pushdecl (x)
 
   return x;
 }
+
 void
 pushdecl_force_head (x)
      tree x;
@@ -1060,8 +1205,7 @@ getdecls ()
 
 /* Create a new `struct binding_level'.  */
 
-static
-struct binding_level *
+static struct binding_level *
 make_binding_level ()
 {
   /* NOSTRICT */
@@ -1104,6 +1248,14 @@ pushlevel (unused)
   keep_next_level_flag = 0;
   newlevel->keep_if_subblocks = keep_next_if_subblocks;
   keep_next_if_subblocks = 0;
+#if defined(DEBUG_JAVA_BINDING_LEVELS)
+  newlevel->binding_depth = binding_depth;
+  indent ();
+  fprintf (stderr, "push %s level 0x%08x pc %d\n",
+	   (is_class_level) ? "class" : "block", newlevel, current_pc);
+  is_class_level = 0;
+  binding_depth++;
+#endif /* defined(DEBUG_JAVA_BINDING_LEVELS) */
 }
 
 /* Exit a binding level.
@@ -1136,6 +1288,26 @@ poplevel (keep, reverse, functionbody)
   tree decl;
   int block_previously_created;
 
+#if defined(DEBUG_JAVA_BINDING_LEVELS)
+  binding_depth--;
+  indent ();
+  if (current_binding_level->end_pc != LARGEST_PC)
+    fprintf (stderr, "pop  %s level 0x%08x pc %d (end pc %d)\n",
+	     (is_class_level) ? "class" : "block", current_binding_level, current_pc,
+	     current_binding_level->end_pc);
+  else
+    fprintf (stderr, "pop  %s level 0x%08x pc %d\n",
+	     (is_class_level) ? "class" : "block", current_binding_level, current_pc);
+#if 0
+  if (is_class_level != (current_binding_level == class_binding_level))
+    {
+      indent ();
+      fprintf (stderr, "XXX is_class_level != (current_binding_level == class_binding_level)\n");
+    }
+  is_class_level = 0;
+#endif
+#endif /* defined(DEBUG_JAVA_BINDING_LEVELS) */
+
   keep |= current_binding_level->keep;
 
   /* Get the decls in the order they were written.
@@ -1188,9 +1360,7 @@ poplevel (keep, reverse, functionbody)
   if (block != 0)
     {
       BLOCK_VARS (block) = decls;
-      BLOCK_TYPE_TAGS (block) = NULL_TREE;
       BLOCK_SUBBLOCKS (block) = subblocks;
-      remember_end_note (block);
     }
 
   /* In each subblock, record that this is its superior.  */
@@ -1253,7 +1423,7 @@ poplevel (keep, reverse, functionbody)
 	      define_label (input_filename, lineno,
 			    DECL_NAME (label));
 	    }
-	  else if (warn_unused && !TREE_USED (label))
+	  else if (warn_unused[UNUSED_LABEL] && !TREE_USED (label))
 	    warning_with_decl (label, "label `%s' defined but not used");
 	  IDENTIFIER_LABEL_VALUE (DECL_NAME (label)) = 0;
 
@@ -1317,6 +1487,10 @@ void
 maybe_pushlevels (pc)
      int pc;
 {
+#if defined(DEBUG_JAVA_BINDING_LEVELS)
+  current_pc = pc;
+#endif
+
   while (pending_local_decls != NULL_TREE &&
 	 DECL_LOCAL_START_PC (pending_local_decls) <= pc)
     {
@@ -1335,25 +1509,56 @@ maybe_pushlevels (pc)
       if (end_pc > current_binding_level->end_pc)
 	end_pc = current_binding_level->end_pc;
 
+      maybe_start_try (pc, end_pc);
+      
       pushlevel (1);
       expand_start_bindings (0);
+
       current_binding_level->end_pc = end_pc;
-      
+      current_binding_level->start_pc = pc;      
       current_binding_level->names = decl;
       for ( ; decl != NULL_TREE;  decl = TREE_CHAIN (decl))
 	{
 	  push_jvm_slot (DECL_LOCAL_SLOT_NUMBER (decl), decl);
 	}
-    }
+    }      
+
+  maybe_start_try (pc, 0);
 }
 
 void
 maybe_poplevels (pc)
      int pc;
 {
+#if defined(DEBUG_JAVA_BINDING_LEVELS)
+  current_pc = pc;
+#endif
+
   while (current_binding_level->end_pc <= pc)
     {
       expand_end_bindings (getdecls (), 1, 0);
+      maybe_end_try (current_binding_level->start_pc, pc);
+      poplevel (1, 0, 0);
+    }
+  maybe_end_try (0, pc);
+}
+
+/* Terminate any binding which began during the range beginning at
+   start_pc.  This tidies up improperly nested local variable ranges
+   and exception handlers; a variable declared within an exception
+   range is forcibly terminated when that exception ends. */
+
+void
+force_poplevels (start_pc)
+     int start_pc;
+{
+  while (current_binding_level->start_pc > start_pc)
+    {
+      if (pedantic && current_binding_level->start_pc > start_pc)
+	warning_with_decl (current_function_decl, 
+			   "In %s: overlapped variable and exception ranges at %d",
+			   current_binding_level->start_pc);
+      expand_end_bindings (getdecls (), 1, 0);
       poplevel (1, 0, 0);
     }
 }
@@ -1367,7 +1572,6 @@ insert_block (block)
      tree block;
 {
   TREE_USED (block) = 1;
-  abort ();
   current_binding_level->blocks
     = chainon (current_binding_level->blocks, block);
 }
@@ -1452,6 +1656,14 @@ give_name_to_locals (jcf)
 	    = (struct lang_decl *) permalloc (sizeof (struct lang_decl_var));
 	  DECL_LOCAL_SLOT_NUMBER (decl) = slot;
 	  DECL_LOCAL_START_PC (decl) = start_pc;
+#if 0
+	  /* FIXME: The range used internally for exceptions and local
+	     variable ranges, is a half-open interval: 
+	     start_pc <= pc < end_pc.  However, the range used in the
+	     Java VM spec is inclusive at both ends: 
+	     start_pc <= pc <= end_pc. */
+	  end_pc++;
+#endif
 	  DECL_LOCAL_END_PC (decl) = end_pc;
 
 	  /* Now insert the new decl in the proper place in
@@ -1503,6 +1715,35 @@ build_result_decl (fndecl)
   return (DECL_RESULT (fndecl) = build_decl (RESULT_DECL, NULL_TREE, restype));
 }
 
+/* Called for every element in DECL_FUNCTION_INIT_TEST_TABLE in order
+   to emit initialization code for each test flag.  */
+
+static boolean
+emit_init_test_initialization (entry, key)
+  struct hash_entry *entry;
+  hash_table_key key ATTRIBUTE_UNUSED;
+{
+  struct init_test_hash_entry *ite = (struct init_test_hash_entry *) entry;
+  tree klass = build_class_ref ((tree) entry->key);
+  expand_decl (ite->init_test_decl);
+
+  /* We initialize the class init check variable by looking at the
+     `state' field of the class to see if it is already initialized.
+     This makes things a bit faster if the class is already
+     initialized, which should be the common case.  */
+  expand_expr_stmt
+    (build (MODIFY_EXPR, boolean_type_node, 
+	    ite->init_test_decl,
+	    build (GE_EXPR, boolean_type_node,
+		   build (COMPONENT_REF, byte_type_node,
+			  build1 (INDIRECT_REF, class_type_node, klass),
+			  lookup_field (&class_type_node,
+					get_identifier ("state"))),
+		   build_int_2 (JV_STATE_DONE, 0))));
+
+  return true;
+}
+
 void
 complete_start_java_method (fndecl)
   tree fndecl;
@@ -1514,6 +1755,11 @@ complete_start_java_method (fndecl)
 
       /* Set up parameters and prepare for return, for the function.  */
       expand_function_start (fndecl, 0);
+
+      /* Emit initialization code for test flags.  */
+      if (! always_initialize_class_p)
+	hash_traverse (&DECL_FUNCTION_INIT_TEST_TABLE (fndecl),
+		       emit_init_test_initialization, 0);
     }
 
   /* Allocate further tree nodes temporarily during compilation
@@ -1530,6 +1776,7 @@ complete_start_java_method (fndecl)
 
   if (METHOD_STATIC (fndecl) && ! METHOD_PRIVATE (fndecl)
       && ! flag_emit_class_files
+      && ! DECL_CLINIT_P (fndecl)
       && ! CLASS_INTERFACE (TYPE_NAME (current_class)))
     {
       tree clas = DECL_CONTEXT (fndecl);
@@ -1553,7 +1800,7 @@ complete_start_java_method (fndecl)
 
   if (METHOD_SYNCHRONIZED (fndecl) && ! flag_emit_class_files)
     {
-      /* Warp function body with a monitorenter plus monitorexit cleanup. */
+      /* Wrap function body with a monitorenter plus monitorexit cleanup. */
       tree enter, exit, lock;
       if (METHOD_STATIC (fndecl))
 	lock = build_class_ref (DECL_CONTEXT (fndecl));
@@ -1596,6 +1843,10 @@ start_java_method (fndecl)
   decl_map = make_tree_vec (i);
   type_map = (tree *) oballoc (i * sizeof (tree));
 
+#if defined(DEBUG_JAVA_BINDING_LEVELS)
+  fprintf (stderr, "%s:\n", (*decl_printable_name) (fndecl, 2));
+  current_pc = 0;
+#endif /* defined(DEBUG_JAVA_BINDING_LEVELS) */
   pushlevel (1);  /* Push parameters. */
 
   ptr = &DECL_ARGUMENTS (fndecl);
@@ -1609,11 +1860,10 @@ start_java_method (fndecl)
 
       parm_decl = build_decl (PARM_DECL, parm_name, parm_type);
       DECL_CONTEXT (parm_decl) = fndecl;
-#ifdef PROMOTE_PROTOTYPES
-      if (TYPE_PRECISION (parm_type) < TYPE_PRECISION (integer_type_node)
+      if (PROMOTE_PROTOTYPES
+	  && TYPE_PRECISION (parm_type) < TYPE_PRECISION (integer_type_node)
 	  && INTEGRAL_TYPE_P (parm_type))
 	parm_type = integer_type_node;
-#endif
       DECL_ARG_TYPE (parm_decl) = parm_type;
 
       *ptr = parm_decl;
