@@ -1,5 +1,5 @@
 /* Utility routines for finding and reading Java(TM) .class files.
-   Copyright (C) 1996, 97-98, 1999  Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997, 1998, 1999, 2000  Free Software Foundation, Inc.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@ The Free Software Foundation is independent of Sun Microsystems, Inc.  */
 
 #include "jcf.h"
 #include "tree.h"
+#include "toplev.h"
 #include "java-tree.h"
 
 /* DOS brain-damage */
@@ -169,8 +170,8 @@ DEFUN(open_in_zip, (jcf, zipfile, zipmember, is_system),
 	  jcf->read_ptr = jcf->buffer;
 	  jcf->read_end = jcf->buffer_end;
 	  jcf->filbuf = jcf_unexpected_eof;
-	  jcf->filename = strdup (zipfile);
-	  jcf->classname = strdup (zipmember);
+	  jcf->filename = xstrdup (zipfile);
+	  jcf->classname = xstrdup (zipmember);
 	  jcf->zipd = (void *)zipd;
 	  if (lseek (zipf->fd, zipd->filestart, 0) < 0
 	      || read (zipf->fd, jcf->buffer, zipd->size) != zipd->size)
@@ -182,9 +183,10 @@ DEFUN(open_in_zip, (jcf, zipfile, zipmember, is_system),
 }
 
 #if JCF_USE_STDIO
-char*
+const char *
 DEFUN(open_class, (filename, jcf, stream, dep_name),
-      char *filename AND JCF *jcf AND FILE* stream AND const char *dep_name)
+      const char *filename AND JCF *jcf AND FILE* stream
+      AND const char *dep_name)
 {
   if (jcf)
     {
@@ -203,9 +205,9 @@ DEFUN(open_class, (filename, jcf, stream, dep_name),
   return filename;
 }
 #else
-char*
+const char *
 DEFUN(open_class, (filename, jcf, fd, dep_name),
-      char *filename AND JCF *jcf AND int fd AND const char *dep_name)
+      const char *filename AND JCF *jcf AND int fd AND const char *dep_name)
 {
   if (jcf)
     {
@@ -240,7 +242,7 @@ DEFUN(open_class, (filename, jcf, fd, dep_name),
 #endif
 
 
-char *
+const char *
 DEFUN(find_classfile, (filename, jcf, dep_name),
       char *filename AND JCF *jcf AND const char *dep_name)
 {
@@ -262,7 +264,7 @@ DEFUN(find_classfile, (filename, jcf, dep_name),
    failure.  If JCF != NULL, it is suitably initialized.
    SOURCE_OK is true if we should also look for .java file. */
 
-char *
+const char *
 DEFUN(find_class, (classname, classname_length, jcf, source_ok),
       const char *classname AND int classname_length AND JCF *jcf AND int source_ok)
 
@@ -286,11 +288,11 @@ DEFUN(find_class, (classname, classname_length, jcf, source_ok),
 
   java_buffer = (char *) alloca (buflen);
 
-  jcf->java_source = jcf->outofsynch = 0;
+  jcf->java_source = 0;
 
   for (entry = jcf_path_start (); entry != NULL; entry = jcf_path_next (entry))
     {
-      char *path_name = jcf_path_name (entry);
+      const char *path_name = jcf_path_name (entry);
       if (class != 0)
 	{
 	  int dir_len;
@@ -352,8 +354,24 @@ DEFUN(find_class, (classname, classname_length, jcf, source_ok),
 	}
     }
 
+  /* We preferably pick a class file if we have a chance. If the source
+     file is newer than the class file, we issue a warning and parse the
+     source file instead.
+     There should be a flag to allow people have the class file picked
+     up no matter what. FIXME. */
   if (! java && ! class && java_buf.st_mtime >= class_buf.st_mtime)
-    jcf->outofsynch = 1;
+    {
+      char *stripped_class_name = xstrdup (classname);
+      int i = strlen (stripped_class_name);
+      
+      while (stripped_class_name [i] != '.')
+	i--;
+      
+      stripped_class_name [i] = '\0';
+      warning ("Source file for class `%s' is newer than its matching class file. Source file used instead", stripped_class_name);
+      free (stripped_class_name);
+      class = -1;
+    }
 
   if (! java)
     dep_file = java_buffer;
@@ -382,7 +400,10 @@ DEFUN(find_class, (classname, classname_length, jcf, source_ok),
 #else
   if (!class)
     {
-      SOURCE_FRONTEND_DEBUG (("Trying %s", buffer));
+      SOURCE_FRONTEND_DEBUG ((stderr, "[Class selected: %s]\n",
+			      classname+classname_length-
+			      (classname_length <= 30 ? 
+			       classname_length : 30)));
       fd = open (buffer, O_RDONLY | O_BINARY);
       if (fd >= 0)
 	goto found;
@@ -391,7 +412,10 @@ DEFUN(find_class, (classname, classname_length, jcf, source_ok),
   if (!java)
     {
       strcpy (buffer, java_buffer);
-      SOURCE_FRONTEND_DEBUG (("Trying %s", buffer));
+      SOURCE_FRONTEND_DEBUG ((stderr, "[Source selected: %s]\n",
+			      classname+classname_length-
+			      (classname_length <= 30 ? 
+			       classname_length : 30)));
       fd = open (buffer, O_RDONLY);
       if (fd >= 0)
 	{
@@ -414,14 +438,12 @@ DEFUN(find_class, (classname, classname_length, jcf, source_ok),
     {
       JCF_ZERO (jcf);		/* JCF_FINISH relies on this */
       jcf->java_source = 1;
-      jcf->filename = (char *) strdup (buffer);
+      jcf->filename = xstrdup (buffer);
       close (fd);		/* We use STDIO for source file */
     }
   else
-    buffer = open_class (buffer, jcf, fd, dep_file);
-  jcf->classname = (char *) ALLOC (classname_length + 1);
-  strncpy (jcf->classname, classname, classname_length + 1);
-  jcf->classname = (char *) strdup (classname);
+    buffer = (char *) open_class (buffer, jcf, fd, dep_file);
+  jcf->classname = xstrdup (classname);
   return buffer;
 #endif
 }
@@ -482,11 +504,15 @@ DEFUN(jcf_print_utf8_replace, (stream, str, length, in_char, out_char),
       FILE *stream AND const unsigned char *str AND int length
       AND int in_char AND int out_char)
 {
-
-  int i;/* FIXME - actually handle Unicode! */
-  for (i = 0; i < length; i++)
+  const unsigned char *limit = str + length;
+  while (str < limit)
     {
-      int ch = str[i];
+      int ch = UTF8_GET (str, limit);
+      if (ch < 0)
+	{
+	  fprintf (stream, "\\<invalid>");
+	  return;
+	}
       jcf_print_char (stream, ch == in_char ? out_char : ch);
     }
 }
