@@ -1,9 +1,9 @@
-static char junk[] = "\n@(#)LIBF77 VERSION 19990503\n";
+static char junk[] = "\n@(#)LIBF77 VERSION 19991115\n";
 
 /*
 */
 
-char __G77_LIBF77_VERSION__[] = "0.5.25 20010315 (release)";
+char __G77_LIBF77_VERSION__[] = "0.5.26 20000731 (prerelease)";
 
 /*
 2.00	11 June 1980.  File version.c added to library.
@@ -61,6 +61,14 @@ char __G77_LIBF77_VERSION__[] = "0.5.25 20010315 (release)";
 			overlapping arguments caused by equivalence.
 	3 May 1999:	"invisible" tweaks to omit compiler warnings in
 			abort_.c, ef1asc_.c, s_rnge.c, s_stop.c.
+	7 Sept. 1999: [cz]_div.c: arrange for compilation under
+			-DIEEE_COMPLEX_DIVIDE to make these routines
+			avoid calling sig_die when the denominator
+			vanishes; instead, they return pairs of NaNs
+			or Infinities, depending whether the numerator
+			also vanishes or not.  VERSION not changed.
+	15 Nov. 1999: s_rnge.c: add casts for the case of
+			sizeof(ftnint) == sizeof(int) < sizeof(long).
 */
 
 #include <stdio.h>
